@@ -28,6 +28,7 @@ ERR_WRONG_SAVE_SIGNAL = -5
 ERR_WRONG_STOP_SIGNAL = -6
 
 
+
 def signalHandler(sig, frame):
     if sig in (signal.SIGINT, signal.SIGTERM):
         if proxy.isRunning():
@@ -41,7 +42,6 @@ def signalHandler(sig, frame):
 
 def ifDebug(string):
     if debug:
-        #print(string, file=sys.stderr)
         sys.stderr.write(string + '\n')
 
 
@@ -79,14 +79,7 @@ class ProxyDialog(QMainWindow):
         self.fields_allow_start = False
         self.process_is_running = False
 
-        self.ui.comboSaveSig.addItem(_translate('proxy', 'None'), 0)
-        self.ui.comboSaveSig.addItem('SIGUSR1', int(signal.SIGUSR1))
-        self.ui.comboSaveSig.addItem('SIGUSR2', int(signal.SIGUSR2))
-        self.ui.comboSaveSig.addItem('SIGINT',  int(signal.SIGINT))
         
-        self.ui.comboStopSig.addItem('SIGTERM', int(signal.SIGTERM))
-        self.ui.comboStopSig.addItem('SIGINT', int(signal.SIGINT))
-        self.ui.comboStopSig.addItem('SIGHUP', int(signal.SIGHUP))
         self.ui.toolButtonBrowse.clicked.connect(self.browseFile)
 
         self.ui.lineEditExecutable.textEdited.connect(
@@ -95,7 +88,30 @@ class ProxyDialog(QMainWindow):
             self.lineEditArgumentsChanged)
         self.ui.lineEditConfigFile.textChanged.connect(
             self.lineEditConfigFileChanged)
-
+        
+        self.ui.comboSaveSig.addItem(_translate('proxy', 'None'), 0)
+        self.ui.comboSaveSig.addItem('SIGUSR1', int(signal.SIGUSR1))
+        self.ui.comboSaveSig.addItem('SIGUSR2', int(signal.SIGUSR2))
+        self.ui.comboSaveSig.addItem('SIGINT',  int(signal.SIGINT))
+        self.ui.comboSaveSig.activated.connect(self.comboSaveSigChanged)
+        self.ui.comboSaveSig.setCurrentIndex(0)
+        
+        self.ui.comboNoSave.addItem(
+            _translate('proxy', "0 - Ignore missing save"))
+        self.ui.comboNoSave.addItem(
+            _translate('proxy', "1 - Transmit missing save"))
+        self.ui.comboNoSave.addItem(
+            _translate('proxy', "2 - Accept windows close"))
+        self.ui.comboNoSave.setCurrentIndex(2)
+        self.ui.comboNoSave.activated.connect(self.comboNoSaveChanged)
+        self.ui.labelNoSaveLevel.setToolTip(self.ui.comboNoSave.toolTip())
+        
+        self.ui.comboStopSig.addItem('SIGTERM', int(signal.SIGTERM))
+        self.ui.comboStopSig.addItem('SIGINT', int(signal.SIGINT))
+        self.ui.comboStopSig.addItem('SIGHUP', int(signal.SIGHUP))
+        self.ui.comboStopSig.activated.connect(self.comboStopSigChanged)
+        self.ui.comboStopSig.setCurrentIndex(0)
+        
         self.ui.comboSaveSig.currentTextChanged.connect(self.allowSaveTest)
         self.ui.toolButtonTestSave.clicked.connect(self.testSave)
         self.ui.toolButtonTestSave.setEnabled(False)
@@ -118,7 +134,7 @@ class ProxyDialog(QMainWindow):
         self.fields_allow_start = True
         if not self.ui.lineEditExecutable.text():
             self.fields_allow_start = False
-
+        
         if ray.shellLineToArgs(self.ui.lineEditArguments.text()) is None:
             self.fields_allow_start = False
 
@@ -126,19 +142,20 @@ class ProxyDialog(QMainWindow):
             bool(not self.process_is_running and self.fields_allow_start))
 
     def updateValuesFromProxyFile(self):
-        proxy_file = proxy.proxy_file
-
-        self.ui.lineEditExecutable.setText(proxy_file.executable)
-        self.ui.lineEditConfigFile.setText(proxy_file.config_file)
-        self.ui.lineEditArguments.setText(proxy_file.arguments_line)
+        self.ui.lineEditExecutable.setText(proxy.executable)
+        self.ui.lineEditConfigFile.setText(proxy.config_file)
+        self.ui.lineEditArguments.setText(proxy.arguments_line)
                 
-        save_index = self.ui.comboSaveSig.findData(proxy_file.save_signal)
+        save_index = self.ui.comboSaveSig.findData(proxy.save_signal)
         self.ui.comboSaveSig.setCurrentIndex(save_index)
 
-        stop_index = self.ui.comboStopSig.findData(proxy_file.stop_signal)
+        self.ui.comboNoSave.setCurrentIndex(proxy.no_save_level)
+        self.ui.comboNoSave.setEnabled(not bool(proxy.save_signal))
+        
+        stop_index = self.ui.comboStopSig.findData(proxy.stop_signal)
         self.ui.comboStopSig.setCurrentIndex(stop_index)
 
-        self.ui.checkBoxWaitWindow.setChecked(proxy_file.wait_window)
+        self.ui.checkBoxWaitWindow.setChecked(proxy.wait_window)
 
         self.checkAllowStart()
 
@@ -169,6 +186,11 @@ class ProxyDialog(QMainWindow):
                     qfile.copy(config_file)
 
         self.config_file = os.path.relpath(config_file)
+        if (proxy.session_name
+            and (self.config_file == proxy.session_name
+                 or self.config_file.startswith("%s." % proxy.session_name))):
+            self.config_file = self.config_file.replace(proxy.session_name,
+                                                        "$RAY_SESSION_NAME")
         self.ui.lineEditConfigFile.setText(self.config_file)
 
     def lineEditExecutableEdited(self, text):
@@ -186,9 +208,40 @@ class ProxyDialog(QMainWindow):
     def lineEditConfigFileChanged(self, text):
         if text and not self.ui.lineEditArguments.text():
             self.ui.lineEditArguments.setText('"$CONFIG_FILE"')
-        elif not text and self.ui.lineEditArguments.text() == '"$CONFIG_FILE"':
+        elif (not text 
+              and self.ui.lineEditArguments.text() == '"$CONFIG_FILE"'):
             self.ui.lineEditArguments.setText('')
 
+    def comboSaveSigChanged(self, index):
+        self.ui.comboNoSave.setEnabled(bool(index == 0))
+        
+        save_signal = 0
+        
+        if index == 1:
+            save_signal = signal.SIGUSR1
+        elif index == 2:
+            save_signal = signal.SIGUSR2
+        elif index == 3:
+            save_signal = signal.SIGINT
+            
+        save_signal = int(save_signal)
+        self.proxy.setSaveSignal(save_signal)
+    
+    def comboStopSigChanged(self, index):
+        stop_signal = signal.SIGTERM
+        
+        if index == 1:
+            stop_signal = signal.SIGINT
+        elif index == 2:
+            stop_signal = signal.SIGTERM
+            
+        stop_signal = int(stop_signal)
+        self.proxy.setStopSignal(stop_signal)
+    
+    def comboNoSaveChanged(self, index):
+        self.proxy.no_save_level = index
+        self.proxy.sendNoSaveLevel()
+    
     def allowSaveTest(self, text=None):
         if text is None:
             text = self.ui.comboSaveSig.currentText()
@@ -200,26 +253,28 @@ class ProxyDialog(QMainWindow):
         save_signal = self.ui.comboSaveSig.currentData()
         proxy.saveProcess(save_signal)
 
-    def saveProxyFile(self):
+    def saveProxy(self):
         executable = self.ui.lineEditExecutable.text()
         config_file = self.ui.lineEditConfigFile.text()
         arguments_line = self.ui.lineEditArguments.text()
         save_signal = self.ui.comboSaveSig.currentData()
+        no_save_level = self.ui.comboNoSave.currentIndex()
         stop_signal = self.ui.comboStopSig.currentData()
         wait_window = self.ui.checkBoxWaitWindow.isChecked()
 
-        proxy.proxy_file.saveFile(
+        proxy.updateAndSave(
             executable,
             config_file,
             arguments_line,
             save_signal,
+            no_save_level,
             stop_signal,
             wait_window)
 
     def startProcess(self):
-        self.saveProxyFile()
+        self.saveProxy()
 
-        if proxy.proxy_file.is_launchable:
+        if proxy.is_launchable:
             proxy.startProcess()
 
     def stopProcess(self):
@@ -262,7 +317,7 @@ class ProxyDialog(QMainWindow):
         settings.sync()
 
         if self.fields_allow_start:
-            self.saveProxyFile()
+            self.saveProxy()
 
         QMainWindow.closeEvent(self, event)
 
@@ -297,21 +352,25 @@ class Proxy(QObject):
         self.process.setProcessChannelMode(QProcess.ForwardedChannels)
         self.process.finished.connect(self.processFinished)
 
-        self.proxy_file = None
-        self.project_path = None
-        self.session_name = None
-        self.full_client_id = None
+        #self.proxy_file = None
+        self.is_launchable = False
+        self.project_path = ""
+        self.path = ""
+        self.session_name = ""
+        self.full_client_id = ""
 
         self.executable = executable
         self.arguments = []
         self.arguments_line = ''
-        self.config_file = None
-        self.save_signal = None
-        self.stop_signal = signal.SIGTERM
-        self.label = None
+        self.config_file = ""
+        self.save_signal = 0
+        self.no_save_level = 2
+        self.stop_signal = int(signal.SIGTERM)
+        self.label = ""
         
         self.wait_window = False
 
+        self._config_file_used = False
         self._wait_for_stop = False
         
         self.timer_save = QTimer()
@@ -347,12 +406,159 @@ class Proxy(QObject):
 
     def waitForStop(self):
         self._wait_for_stop = True
+    
+    def setSaveSignal(self, int_signal):
+        self.save_signal = int_signal
+        self.sendNoSaveLevel()
         
+    def setStopSignal(self, int_signal):
+        self.stop_signal = int_signal
+    
+    def readFile(self):
+        self.is_launchable = False
+        try:
+            file = open(self.path, 'r')
+        except BaseException:
+            return
+
+        xml = QDomDocument()
+        xml.setContent(file.read())
+
+        content = xml.documentElement()
+
+        if content.tagName() != "RAY-PROXY":
+            file.close()
+            return
+
+        cte = content.toElement()
+        file_version = cte.attribute('VERSION')
+        self.executable = cte.attribute('executable')
+        self.config_file = cte.attribute('config_file')
+        self.arguments_line = cte.attribute('arguments')
+        save_signal = cte.attribute('save_signal')
+        no_save_level = cte.attribute('no_save_level')
+        stop_signal = cte.attribute('stop_signal')
+
+        wait_window = cte.attribute('wait_window')
+
+        if wait_window.isdigit():
+            self.wait_window = bool(int(wait_window))
+        else:
+            self.wait_window = False
+
+        file.close()
+
+        if save_signal.isdigit():
+            self.save_signal = int(save_signal)
+        
+        if no_save_level.isdigit():
+            self.no_save_level = int(no_save_level)
+        else:
+            self.no_save_level = 2
+        
+        versions = [file_version, '0.7.1']
+        versions.sort()
+        
+        if file_version != versions[0]:
+            # something was wrong in old version,
+            # save signal was saved as stop signal too.
+            # so don't read stop signal if this is an old file.
+            if stop_signal.isdigit():
+                self.stop_signal = int(stop_signal)
+
+        if not self.executable:
+            return
+
+        arguments = ray.shellLineToArgs(self.arguments_line)
+        if arguments is None:
+            return
+
+        self.is_launchable = True
+
+    def saveFile(
+            self,
+            executable,
+            config_file,
+            arguments_line,
+            save_signal,
+            no_save_level,
+            stop_signal,
+            wait_window):
+        try:
+            file = open(self.path, 'w')
+        except BaseException:
+            return
+
+        if not save_signal:
+            save_signal = 0
+
+        xml = QDomDocument()
+        p = xml.createElement('RAY-PROXY')
+        p.setAttribute('VERSION', ray.VERSION)
+        p.setAttribute('executable', executable)
+        p.setAttribute('arguments', arguments_line)
+        p.setAttribute('config_file', config_file)
+        p.setAttribute('save_signal', str(int(save_signal)))
+        p.setAttribute('no_save_level', str(no_save_level))
+        p.setAttribute('stop_signal', str(int(stop_signal)))
+        p.setAttribute('wait_window', wait_window)
+
+        xml.appendChild(p)
+
+        contents = "<?xml version='1.0' encoding='UTF-8'?>\n"
+        contents += "<!DOCTYPE RAY-PROXY>\n"
+        contents += xml.toString()
+
+        file.write(contents)
+        file.close()
+
+        self.readFile()
+    
+    def updateValues(self, executable, config_file, arguments_line,
+                     save_signal, no_save_level, stop_signal, wait_window):
+        self.executable = executable
+        self.config_file = config_file
+        self.arguments_line = arguments_line
+        self.save_signal = save_signal
+        self.no_save_level = no_save_level
+        self.stop_signal = stop_signal
+        self.wait_window = wait_window
+    
+    def saveProxyFile(self):
+        self.saveFile(
+            self.executable,
+            self.config_file,
+            self.arguments_line,
+            self.save_signal,
+            self.no_save_level,
+            self.stop_signal,
+            self.wait_window)
+    
+    def updateAndSave(self, executable, config_file, arguments_line,
+                      save_signal, no_save_level, stop_signal, wait_window):
+        self.updateValues(executable, config_file, arguments_line, 
+                          save_signal, no_save_level,
+                          stop_signal, wait_window)
+        self.saveProxyFile()
+    
+    def processFinished(self, exit_code):
+        if self._wait_for_stop:
+            app.quit()
+            
+        if self.is_finishable:
+            if not proxy_dialog.isVisible():
+                app.quit()
+        else:
+            duration = time.time() - self.process_start_time
+            proxy_dialog.processTerminateShortly(duration)
+            # proxy_dialog.show()
+    
     def checkWindow(self):
         self.timer_window_n += 1
 
         if self.timer_window_n > 600:
-            # 600 x 50ms = 30s max until ray-proxy replyOpen to Session Manager
+            # 600 x 50ms = 30s max until ray-proxy 
+            # replyOpen to Session Manager
             self.checkWindowEnded()
             return
 
@@ -417,7 +623,6 @@ class Proxy(QObject):
                     self.checkWindowEnded()
                     return
                 
-
             if ppid == parent_pid:
                 # a window appears with a pid child of this ray-proxy,
                 # replyOpen
@@ -428,19 +633,6 @@ class Proxy(QObject):
         self.timer_window.stop()
         server.openReply()
 
-    def processFinished(self, exit_code):
-        if self._wait_for_stop:
-            app.quit()
-            
-        if self.is_finishable:
-            if not proxy_dialog.isVisible():
-                #sys.exit(0)
-                app.quit()
-        else:
-            duration = time.time() - self.process_start_time
-            proxy_dialog.processTerminateShortly(duration)
-            # proxy_dialog.show()
-
     def initialize(self, project_path, session_name, full_client_id):
         self.project_path = project_path
         self.session_name = session_name
@@ -453,40 +645,51 @@ class Proxy(QObject):
 
         os.chdir(project_path)
 
-        proxy_dialog.setWindowTitle(self.full_client_id)
-
-        self.proxy_file = ProxyFile(project_path, self.executable)
-        self.proxy_file.readFile()
+        #proxy_dialog.setWindowTitle(self.full_client_id)
+        
+        self.path = os.path.join(project_path, "ray-proxy.xml")
+        self.readFile()
 
         proxy_dialog.updateValuesFromProxyFile()
 
-        if not self.proxy_file.is_launchable:
+        if not self.is_launchable:
             server.openReply()
             proxy_dialog.show()
             return
 
         self.startProcess()
-
+    
+    def sendNoSaveLevel(self):
+        if ':no-save-level:' in server.getServerCapabilities():
+            nsl = self.no_save_level
+            if self.save_signal:
+                nsl = 0
+            server.sendToDaemon('/nsm/client/no_save_level', nsl)
+    
     def startProcess(self):
         os.environ['NSM_CLIENT_ID'] = self.full_client_id
         os.environ['RAY_SESSION_NAME'] = self.session_name
 
         # enable environment vars in config_file
-        config_file = os.path.expandvars(self.proxy_file.config_file)
+        config_file = os.path.expandvars(self.config_file)
         os.environ['CONFIG_FILE'] = config_file
 
-        # Useful for launching with specifics arguments clients NSM compatible
+        # Useful for launching NSM compatible clients with specifics arguments
         os.unsetenv('NSM_URL')
 
-        arguments_line = os.path.expandvars(self.proxy_file.arguments_line)
+        arguments_line = os.path.expandvars(self.arguments_line)
         arguments = ray.shellLineToArgs(arguments_line)
-
-        self.process.start(self.proxy_file.executable, arguments)
+        
+        self._config_file_used = bool(config_file
+                                      and config_file in arguments)
+        self.sendNoSaveLevel()
+        
+        self.process.start(self.executable, arguments)
         self.timer_open.start()
 
     def saveProcess(self, save_signal=0):
         if not save_signal:
-            save_signal = self.proxy_file.save_signal
+            save_signal = self.save_signal
 
         if self.isRunning() and save_signal:
             os.kill(self.process.processId(), save_signal)
@@ -506,7 +709,7 @@ class Proxy(QObject):
         server.saveReply()
 
     def timerOpenFinished(self):
-        if self.proxy_file.wait_window:
+        if self.wait_window:
             self.timer_window.start()
         else:
             server.openReply()
@@ -531,8 +734,7 @@ class Proxy(QObject):
 
 class ProxyFile(object):
     def __init__(self, project_path, executable=''):
-        self.file = None
-        self.path = "%s/ray-proxy.xml" % project_path
+        
 
         self.executable = executable
         self.arguments_line = ''
@@ -542,100 +744,10 @@ class ProxyFile(object):
         self.stop_signal = int(signal.SIGTERM)
         self.wait_window = False
 
-        self.is_launchable = False
-
-    def readFile(self):
-        self.is_launchable = False
-        try:
-            file = open(self.path, 'r')
-        except BaseException:
-            return
-
-        xml = QDomDocument()
-        xml.setContent(file.read())
-
-        content = xml.documentElement()
-
-        if content.tagName() != "RAY-PROXY":
-            file.close()
-            return
-
-        cte = content.toElement()
-        file_version = cte.attribute('VERSION')
-        self.executable = cte.attribute('executable')
-        self.config_file = cte.attribute('config_file')
-        self.arguments_line = cte.attribute('arguments')
-        save_signal = cte.attribute('save_signal')
-        stop_signal = cte.attribute('stop_signal')
-
-        wait_window = cte.attribute('wait_window')
-
-        if wait_window.isdigit():
-            self.wait_window = bool(int(wait_window))
-        else:
-            self.wait_window = False
-
-        file.close()
-
-        if save_signal.isdigit():
-            self.save_signal = int(save_signal)
-            
-        versions = [file_version, '0.7.1']
-        versions.sort()
         
-        if file_version != versions[0]:
-            # something was wrong in old version,
-            # save signal was saved as stop signal too.
-            # so don't read stop signal if this is an old file.
-            if stop_signal.isdigit():
-                self.stop_signal = int(stop_signal)
-
-        if not self.executable:
-            return
-
-        arguments = ray.shellLineToArgs(self.arguments_line)
-        if arguments is None:
-            return
-
-        self.is_launchable = True
-
-    def saveFile(
-            self,
-            executable,
-            config_file,
-            arguments_line,
-            save_signal,
-            stop_signal,
-            wait_window):
-        try:
-            file = open(self.path, 'w')
-        except BaseException:
-            return
-
-        if not save_signal:
-            save_signal = 0
-
-        xml = QDomDocument()
-        p = xml.createElement('RAY-PROXY')
-        p.setAttribute('VERSION', ray.VERSION)
-        p.setAttribute('executable', executable)
-        p.setAttribute('arguments', arguments_line)
-        p.setAttribute('config_file', config_file)
-        p.setAttribute('save_signal', str(int(save_signal)))
-        p.setAttribute('stop_signal', str(int(stop_signal)))
-        p.setAttribute('wait_window', wait_window)
-
-        xml.appendChild(p)
-
-        contents = "<?xml version='1.0' encoding='UTF-8'?>\n"
-        contents += "<!DOCTYPE RAY-PROXY>\n"
-        contents += xml.toString()
-
-        file.write(contents)
-        file.close()
-
-        self.readFile()
 
+    
+    
 
 if __name__ == '__main__':
     NSM_URL = os.getenv('NSM_URL')
@@ -697,7 +809,7 @@ if __name__ == '__main__':
     proxy_dialog = ProxyDialog()
 
     
-    server.announce('Ray Proxy', ':optional-gui:', 'ray-proxy')
+    server.announce('Ray Proxy', ':optional-gui:warning-no-save:', 'ray-proxy')
 
     app.exec()
 
