@@ -49,11 +49,13 @@ class Client(ServerSender):
     tmp_arguments    = ''
     label            = ''
     icon             = ''
-    project_path     = ""
+    custom_prefix    = ""
     prefix_mode      = ray.PrefixMode.SESSION_NAME
     auto_start       = True
     start_gui_hidden = False
     check_last_save  = True
+    no_save_level    = 0
+    is_external      = False
     sent_to_gui      = False
     
     net_session_template = ''
@@ -61,6 +63,8 @@ class Client(ServerSender):
     net_daemon_url       = ''
     net_duplicate_state  = -1
     
+    ignored_extensions = ray.getGitIgnoredExtensions()
+    
     last_save_time = 0.00
     last_dirty = 0.00
     
@@ -118,15 +122,28 @@ class Client(ServerSender):
         self.check_last_save  = bool(ctx.attribute('check_last_save') != '0')
         self.start_gui_hidden = bool(ctx.attribute('gui_visible') == '0')
         
+        ign_exts = ctx.attribute('ignored_extensions').split(' ')
+        unign_exts = ctx.attribute('unignored_extensions').split(' ')
+        
+        global_exts = ray.getGitIgnoredExtensions().split(' ')
+        self.ignored_extensions = ""
+        
+        for ext in global_exts:
+            if ext and not ext in unign_exts:
+                self.ignored_extensions+= " %s" % ext
+                
+        for ext in ign_exts:
+            if ext and not ext in global_exts:
+                self.ignored_extensions+= " %s" % ext
+                
         prefix_mode = ctx.attribute('prefix_mode')
         
-        if prefix_mode and prefix_mode.isdigit():
-            if 0 <= int(prefix_mode) <= 2:
-                self.prefix_mode = int(prefix_mode)
-                if self.prefix_mode == 0:
-                    self.project_path = ctx.attribute('project-path')
+        if (prefix_mode and prefix_mode.isdigit()
+                and 0 <= int(prefix_mode) <= 2 ):
+            self.prefix_mode = int(prefix_mode)
+            if self.prefix_mode == ray.PrefixMode.CUSTOM:
+                self.custom_prefix = ctx.attribute('custom_prefix')
                     
-        
         self.net_session_template = ctx.attribute('net_session_template')
         
         if basename(self.executable_path) == 'ray-network':
@@ -169,15 +186,15 @@ class Client(ServerSender):
         if self.icon:
             ctx.setAttribute('icon', self.icon)
         if not self.check_last_save:
-            ctx.setAttribute('check_last_save', "0")
+            ctx.setAttribute('check_last_save', 0)
         if self.arguments:
             ctx.setAttribute('arguments', self.arguments)
             
         if self.prefix_mode != ray.PrefixMode.SESSION_NAME:
             ctx.setAttribute('prefix_mode', self.prefix_mode)
             
-            if self.prefix_mode == ray.PrefixMode.UNDEF:
-                ctx.setAttribute('project_path', self.project_path)
+            if self.prefix_mode == ray.PrefixMode.CUSTOM:
+                ctx.setAttribute('custom_prefix', self.custom_prefix)
                 
         if self.isCapableOf(':optional-gui:'):
             if self.executable_path != 'ray-proxy':
@@ -188,7 +205,31 @@ class Client(ServerSender):
             ctx.setAttribute('net_session_template',
                              self.net_session_template)
             
-        
+        if self.ignored_extensions != ray.getGitIgnoredExtensions():
+            ignored = ""
+            unignored = ""
+            client_exts = [e for e in self.ignored_extensions.split(' ') if e]
+            global_exts = [e for e in 
+                           ray.getGitIgnoredExtensions().split(' ') if e]
+            
+            for cext in client_exts:
+                if not cext in global_exts:
+                    ignored += " %s" % cext
+            
+            for gext in global_exts:
+                if not gext in client_exts:
+                    unignored += " %s" % gext
+                    
+            if ignored:        
+                ctx.setAttribute('ignored_extensions', ignored)
+            else:
+                ctx.removeAttribute('ignored_extensions')
+                
+            if unignored:
+                ctx.setAttribute('unignored_extensions', unignored)
+            else:
+                ctx.removeAttribute('unignored_extensions')
+                
     def setReply(self, errcode, message):
         self._reply_message = message
         self._reply_errcode = errcode
@@ -202,8 +243,7 @@ class Client(ServerSender):
         self.sendGuiClientProperties()
         
     def hasError(self):
-        if self._reply_errcode:
-            return self._reply_errcode
+        return bool(self._reply_errcode)
         
     def errorCode(self):
         return self._reply_errcode
@@ -212,8 +252,7 @@ class Client(ServerSender):
         return self._reply_message
     
     def isReplyPending(self):
-        if self.pending_command:
-            return self.pending_command
+        return bool(self.pending_command)
         
     def isDumbClient(self):
         return bool(not self.did_announce)
@@ -258,7 +297,7 @@ class Client(ServerSender):
         
         if (status == ray.ClientStatus.COPY
             or self.session.file_copier.isActive(self.client_id)):
-                self.sendGui("/ray/client/status", self.client_id, 
+                self.sendGui("/ray/gui/client/status", self.client_id, 
                              ray.ClientStatus.COPY)
     
     def getJackClientName(self):
@@ -273,24 +312,82 @@ class Client(ServerSender):
         
         return jack_client_name
     
+    def getPrefixString(self):
+        if self.prefix_mode == ray.PrefixMode.SESSION_NAME:
+            return self.session.name
+        
+        elif self.prefix_mode == ray.PrefixMode.CLIENT_NAME:
+            return self.name
+        
+        elif self.prefix_mode == ray.PrefixMode.CUSTOM:
+            return self.custom_prefix
+        
+        return ''
+    
     def getProjectPath(self):
         if self.executable_path == 'ray-network':
-            #for ray-network, use project_path for template,
-            #quite ugly but simple code.
+            # for ray-network, use custom_prefix for template,
+            # quite ugly but simple code.
             return self.net_session_template
         
         if self.prefix_mode == ray.PrefixMode.SESSION_NAME:
             return "%s/%s.%s" % (self.session.path, self.session.name, 
                                  self.client_id)
+        
         elif self.prefix_mode == ray.PrefixMode.CLIENT_NAME:
             return "%s/%s.%s" % (self.session.path, self.name, self.client_id)
-        else:
-            current_dir = os.getcwd()
-            os.chdir(self.session.path)
-            project_path = os.path.realpath(self.project_path)
-            os.chdir(current_dir)
+        
+        elif self.prefix_mode == ray.PrefixMode.CUSTOM:
+            return "%s/%s.%s" % (self.session.path, self.custom_prefix,
+                                 self.client_id)
+        # should not happens
+        return "%s/%s.%s" % (self.session.path, self.session.name,
+                             self.client_id)
+    
+    def getProxyExecutable(self):
+        if os.path.basename(self.executable_path) != 'ray-proxy':
+            return ""
+        
+        xml_file = "%s/ray-proxy.xml" % self.getProjectPath()
+        xml = QDomDocument()
+        try:
+            file = open(xml_file, 'r')
+            xml.setContent(file.read())
+        except:
+            return ""
+        
+        content = xml.documentElement()
+        if content.tagName() != "RAY-PROXY":
+            file.close()
+            return ""
             
-            return project_path
+        executable = content.attribute('executable')
+        file.close()
+        
+        return executable
+    
+    def setDefaultGitIgnored(self, executable=""):
+        executable = executable if executable else self.executable_path
+        executable = os.path.basename(executable)
+        if executable == 'ray-proxy':
+            executable = self.getProxyExecutable()
+        
+        if executable in (
+                'ardour', 'ardour4', 'ardour5', 'ardour6',
+                'Ardour', 'Ardour4', 'Ardour5', 'Ardour6',
+                'qtractor'):
+            self.ignored_extensions += " .mid"
+            
+        elif executable in ('luppp', 'sooperlooper', 'sooperlooper_nsm'):
+            if '.wav' in self.ignored_extensions:
+                self.ignored_extensions = \
+                    self.ignored_extensions.replace('.wav', '')
+                
+        elif executable == 'samplv1_jack':
+            for ext in ('.wav', '.flac', '.ogg', '.mp3'):
+                if ext in self.ignored_extensions:
+                    self.ignored_extensions = \
+                        self.ignored_extensions.replace(ext, '')
     
     def start(self):
         self.session.setRenameable(False)
@@ -321,6 +418,9 @@ class Client(ServerSender):
         self.running_arguments  = self.arguments
         
         self.process.start(self.executable_path, arguments)
+        
+        ## Here for another way to debug clients.
+        ## Konsole is a terminal software.
         #self.process.start(
             #'konsole', 
             #['--hide-tabbar', '--hide-menubar', '-e', self.executable_path]
@@ -328,13 +428,22 @@ class Client(ServerSender):
      
     def terminate(self):
         if self.isRunning():
-            self.process.terminate()
+            if self.is_external:
+                os.kill(self.pid, 15) # 15 means signal.SIGTERM
+            else:
+                self.process.terminate()
         
     def kill(self):
+        if self.is_external:
+            os.kill(self.pid, 9) # 9 means signal.SIGKILL
+            return
+            
         if self.isRunning():
             self.process.kill()
             
     def isRunning(self):
+        if self.is_external:
+            return True
         return bool(self.process.state() == 2)
     
     def standardError(self):
@@ -359,6 +468,7 @@ class Client(ServerSender):
     
     def processFinished(self, exit_code, exit_status):
         self.stopped_timer.stop()
+        self.is_external = False
         
         if self.pending_command in (ray.Command.KILL, ray.Command.QUIT):
             self.sendGuiMessage(_translate('GUIMSG', 
@@ -379,9 +489,9 @@ class Client(ServerSender):
             self.setStatus(ray.ClientStatus.STOPPED)
                 
         self.pending_command = ray.Command.NONE
-        self.active          = False
-        self.pid             = 0
-        self.addr            = None
+        self.active = False
+        self.pid = 0
+        self.addr = None
         
         self.session.setRenameable(True)
         
@@ -389,26 +499,28 @@ class Client(ServerSender):
         if error == QProcess.FailedToStart:
             self.sendGuiMessage(_translate('GUIMSG', "%s Failed to start !") 
                                 % self.guiMsgStyle())
-            self.active     = False
-            self.pid        = 0
+            self.active = False
+            self.pid    = 0
             self.setStatus(ray.ClientStatus.STOPPED)
             self.pending_command = ray.Command.NONE
             
-            #if self.session.osc_src_addr:
-                ##Finally, not sure that it's a good idea to display error 
-                ##dialog in this case.
-                #self.session.oscReply("/error", self.session.osc_path, 
-                                      #ray.Err.LAUNCH_FAILED, 
-                                      #"Failed to launch process!")
+            if self.session.osc_src_addr:
+                error_message = "Failed to launch process!"
+                if not self.session.osc_path.startswith('/nsm/server/'):
+                    error_message = _translate('client',
+                                               "Failed to launch process !")
+                    
+                self.session.oscReply("/error", self.session.osc_path, 
+                                      ray.Err.LAUNCH_FAILED, 
+                                      error_message)
             
             if self.session.wait_for:
                 self.session.endTimerIfLastExpected(self)
-        
         self.session.setRenameable(True)
     
     def stoppedSinceLong(self):
         self.stopped_since_long = True
-        self.sendGui('/ray/client/still_running', self.client_id)
+        self.sendGui('/ray/gui/client/still_running', self.client_id)
     
     def tellClientSessionIsLoaded(self):
         if self.active and not self.isDumbClient():
@@ -416,9 +528,12 @@ class Client(ServerSender):
                              % self.name)
             self.sendToSelfAddress("/nsm/client/session_is_loaded")
     
+    def canSaveNow(self):
+        return bool(self.active and not self.no_save_level) 
+    
     def save(self):
         if self.isRunning():
-            if self.active:
+            if self.canSaveNow():
                 Terminal.message("Telling %s to save" % self.name)
                 self.sendToSelfAddress("/nsm/client/save")
                 
@@ -434,6 +549,11 @@ class Client(ServerSender):
     def stop(self):
         self.sendGuiMessage(_translate('GUIMSG', "%s stopping")
                             % self.guiMsgStyle())
+        
+        #if self.is_external:
+            #os.kill(self.pid, 15) # 15 means signal.SIGTERM
+            #return
+            
         if self.isRunning():
             self.pending_command = ray.Command.KILL
             self.setStatus(ray.ClientStatus.QUIT)
@@ -441,7 +561,7 @@ class Client(ServerSender):
             if not self.stopped_timer.isActive():
                 self.stopped_timer.start()
                 
-            self.process.terminate()
+            self.terminate()
     
     def quit(self):
         Terminal.message("Commanding %s to quit" % self.name)
@@ -450,17 +570,17 @@ class Client(ServerSender):
             self.terminate()
             self.setStatus(ray.ClientStatus.QUIT)
         else:
-            self.sendGui("/ray/client/status", self.client_id, 
+            self.sendGui("/ray/gui/client/status", self.client_id, 
                          ray.ClientStatus.REMOVED)
     
     def switch(self, new_client):
-        old_client_id     = self.client_id
-        self.client_id    = new_client.client_id
-        self.name         = new_client.name
-        self.prefix_mode  = new_client.prefix_mode
-        self.project_path = new_client.project_path
-        self.label        = new_client.label
-        self.icon         = new_client.icon
+        old_client_id      = self.client_id
+        self.client_id     = new_client.client_id
+        self.name          = new_client.name
+        self.prefix_mode   = new_client.prefix_mode
+        self.custom_prefix = new_client.custom_prefix
+        self.label         = new_client.label
+        self.icon          = new_client.icon
         
         jack_client_name    = self.getJackClientName()
         client_project_path = self.getProjectPath()
@@ -474,13 +594,13 @@ class Client(ServerSender):
         self.pending_command = ray.Command.OPEN
         self.setStatus(ray.ClientStatus.SWITCH)
             
-        self.sendGui("/ray/client/switch", old_client_id, self.client_id)
+        self.sendGui("/ray/gui/client/switch", old_client_id, self.client_id)
     
     def sendGuiClientProperties(self, removed=False):
-        ad = '/ray/client/update' if self.sent_to_gui else '/ray/client/new'
+        ad = '/ray/gui/client/update' if self.sent_to_gui else '/ray/gui/client/new'
             
         if removed:
-            ad = '/ray/trash/add'
+            ad = '/ray/gui/trash/add'
             
         self.sendGui(ad,
                         self.client_id, 
@@ -488,11 +608,12 @@ class Client(ServerSender):
                         self.arguments,
                         self.name, 
                         self.prefix_mode, 
-                        self.project_path,
+                        self.custom_prefix,
                         self.label,
                         self.icon,
                         self.capabilities,
-                        int(self.check_last_save))
+                        int(self.check_last_save),
+                        self.ignored_extensions)
         
         self.sent_to_gui = True
     
@@ -501,11 +622,12 @@ class Client(ServerSender):
         self.executable_path = client_data.executable_path
         self.arguments       = client_data.arguments
         self.prefix_mode     = client_data.prefix_mode
-        self.project_path    = client_data.project_path
+        self.custom_prefix   = client_data.custom_prefix
         self.label           = client_data.label
         self.icon            = client_data.icon
         self.capabilities    = client_data.capabilities
         self.check_last_save = client_data.check_last_save
+        self.ignored_extensions = client_data.ignored_extensions
         
         self.sendGuiClientProperties()
     
@@ -567,41 +689,40 @@ class Client(ServerSender):
             
     def saveAsTemplate(self, template_name):
         #copy files
-        if self.prefix_mode != ray.PrefixMode.UNDEF:
-            client_files = self.getProjectFiles()
-                        
-            template_dir = "%s/%s" % (TemplateRoots.user_clients, 
-                                      template_name)
-            
-            if os.path.exists(template_dir):
-                if os.access(template_dir, os.W_OK):
-                    shutil.rmtree(template_dir)
-                else:
-                    #TODO send error
-                    return
-                
-            os.makedirs(template_dir)
-            
-            if self.net_daemon_url:
-                self.net_session_template = template_name
-                self.send(Address(self.net_daemon_url), 
-                          '/ray/session/save_as_template', self.session.name, 
-                          template_name, self.net_session_root)
-            
-            if client_files:
-                self.setStatus(ray.ClientStatus.COPY)
-                fc = self.session.file_copier
-                fc.startClientCopy(self.client_id, client_files, template_dir,
-                                   self.saveAsTemplate_step1,
-                                   self.saveAsTemplateAborted,
-                                   [template_name])
+        client_files = self.getProjectFiles()
+                    
+        template_dir = "%s/%s" % (TemplateRoots.user_clients, 
+                                    template_name)
+        
+        if os.path.exists(template_dir):
+            if os.access(template_dir, os.W_OK):
+                shutil.rmtree(template_dir)
             else:
-                self.saveAsTemplate_step1(template_name)
+                #TODO send error
+                return
+            
+        os.makedirs(template_dir)
+        
+        if self.net_daemon_url:
+            self.net_session_template = template_name
+            self.send(Address(self.net_daemon_url), 
+                        '/ray/session/save_as_template', self.session.name, 
+                        template_name, self.net_session_root)
+        
+        if client_files:
+            self.setStatus(ray.ClientStatus.COPY)
+            fc = self.session.file_copier
+            fc.startClientCopy(self.client_id, client_files, template_dir,
+                                self.saveAsTemplate_step1,
+                                self.saveAsTemplateAborted,
+                                [template_name])
+        else:
+            self.saveAsTemplate_step1(template_name)
 
     def saveAsTemplate_step1(self, template_name):
         self.setStatus(self.status) #see setStatus to see why
         
-        if self.prefix_mode != ray.PrefixMode.UNDEF:
+        if self.prefix_mode != ray.PrefixMode.CUSTOM:
             self.adjustFilesAfterCopy(template_name, ray.Template.CLIENT_SAVE)
             
         xml_file = "%s/%s" % (TemplateRoots.user_clients,
@@ -610,13 +731,13 @@ class Client(ServerSender):
         #security check
         if os.path.exists(xml_file):
             if not os.access(xml_file, os.W_OK):
+                # TODO send error
                 return
             
             if os.path.isdir(xml_file):
                 #should not be a dir, remove it !
                 subprocess.run('rm', '-R', xml_file)
         
-        
         if not os.path.isdir(TemplateRoots.user_clients):
             os.makedirs(TemplateRoots.user_clients)
         
@@ -640,8 +761,7 @@ class Client(ServerSender):
         if not content.tagName() == 'RAY-CLIENT-TEMPLATES':
             return
         
-        
-        #remove existing template if it has the same name as the new one
+        # remove existing template if it has the same name as the new one
         node = content.firstChild()
         while not node.isNull():
             if node.toElement().tagName() != 'Client-Template':
@@ -668,6 +788,10 @@ class Client(ServerSender):
         file = open(xml_file, 'w')
         file.write(xml.toString())
         file.close()
+        
+        self.sendGuiMessage(
+            _translate('message', 'Client template %s created')
+                % template_name)
     
     def saveAsTemplateAborted(self, template_name):
         self.setStatus(self.status)
@@ -842,7 +966,6 @@ class Client(ServerSender):
                                                      config_file.replace(
                                                         old_session_name, 
                                                         new_session_name)))
-                                            
                 file.close()
                         
             except:
@@ -857,6 +980,8 @@ class Client(ServerSender):
         client_name, capabilities, executable_path, major, minor, pid = args
         
         if self.pending_command in (ray.Command.QUIT, ray.Command.KILL):
+            # assume to not answer to a dying client.
+            # He will never know, or perhaps, it depends on beliefs.
             return
         
         if major > NSM_API_VERSION_MAJOR:
@@ -864,7 +989,7 @@ class Client(ServerSender):
                 "Client is using incompatible and more recent " 
                 + "API version %i.%i" % (major, minor))
             self.send(src_addr, "/error", path, ray.Err.INCOMPATIBLE_API, 
-                      "Server is using an incompatible API version." )
+                      "Server is using an incompatible API version.")
             return
         
         self.capabilities = capabilities
@@ -873,6 +998,11 @@ class Client(ServerSender):
         self.active       = True
         self.did_announce = True
         
+        if is_new:
+            self.is_external = True
+            self.pid = pid
+            self.running_executable = executable_path
+        
         if self.executable_path in RS.non_active_clients:
             RS.non_active_clients.remove(self.executable_path)
         
@@ -881,10 +1011,23 @@ class Client(ServerSender):
             "The client \"%s\" at \"%s\" " % (self.name, self.addr.url)
             + "informs us it's ready to receive commands.")
         
+        server = self.getServer()
+        if not server:
+            return 
+        
+        # if this daemon is under another NSM session
+        # do not enable server-control
+        # because new, open and duplicate are forbidden 
+        server_capabilities = "" 
+        if not server.is_nsm_locked:
+            server_capabilities += ":server-control"
+        server_capabilities += ":broadcast:optional-gui:no-save-level:"
+        
         self.send(src_addr, "/reply", path, 
                   "Well hello, stranger. Welcome to the party."
                   if is_new else "Howdy, what took you so long?",
-                  ray.APP_TITLE, ":server-control:broadcast:optional-gui:" )
+                  ray.APP_TITLE,
+                  server_capabilities)
         
         self.sendGuiClientProperties()
         self.setStatus(ray.ClientStatus.OPEN)
@@ -898,7 +1041,7 @@ class Client(ServerSender):
         self.pending_command = ray.Command.OPEN
         
         if self.isCapableOf(":optional-gui:"):
-            self.sendGui("/ray/client/has_optional_gui", self.client_id)
+            self.sendGui("/ray/gui/client/has_optional_gui", self.client_id)
             
             if self.start_gui_hidden:
                 self.send(src_addr, "/nsm/client/hide_optional_gui")
