@@ -1,32 +1,38 @@
 import os
 import sys
 import time
-from PyQt5.QtWidgets import (QDialog, QDialogButtonBox, QListWidgetItem,
-                             QCompleter, QMessageBox, QFileDialog)
-from PyQt5.QtGui import QIcon
-from PyQt5.QtCore import Qt, QTimer, QSettings
+from PyQt5.QtWidgets import (
+    QDialog, QDialogButtonBox, QTreeWidgetItem,
+    QCompleter, QMessageBox, QFileDialog, QWidget)
+from PyQt5.QtGui import QIcon, QPixmap
+from PyQt5.QtCore import Qt, QTimer
 
 import ray
 from gui_server_thread import GUIServerThread
 from gui_tools import (default_session_root, ErrDaemon, _translate,
-                       CommandLineArgs, RS)
+                       CommandLineArgs, RS, isDarkTheme)
 
 import ui_open_session
 import ui_new_session
+import ui_list_snapshots
 import ui_save_template_session
 import ui_nsm_open_info
 import ui_abort_session
 import ui_about_raysession
 import ui_add_application
+import ui_donations
 import ui_new_executable
 import ui_error_dialog
 import ui_quit_app
 import ui_client_properties
 import ui_stop_client
+import ui_stop_client_no_save
 import ui_abort_copy
 import ui_client_trash
 import ui_daemon_url
 import ui_edit_executable
+import ui_snapshot_progress
+import ui_waiting_close_user
 
 class ChildDialog(QDialog):
     def __init__(self, parent):
@@ -105,25 +111,91 @@ class ChildDialog(QDialog):
         QDialog.enterEvent(self, event)
 
 
+class SessionItem(QTreeWidgetItem):
+    def __init__(self, list):
+        QTreeWidgetItem.__init__(self, list)
+        
+    def showConditionnaly(self, string):
+        show = bool(string.lower() in self.data(0, Qt.UserRole).lower())
+        
+        n=0
+        for i in range(self.childCount()):
+            if self.child(i).showConditionnaly(string.lower()):
+                n+=1
+        if n:
+            show = True
+            
+        self.setExpanded(bool(n and string))
+        self.setHidden(not show)
+        return show
+    
+    def findItemWith(self, string):
+        if self.data(0, Qt.UserRole) == string:
+            return self
+        
+        item = None
+        
+        for i in range(self.childCount()):
+            item = self.child(i).findItemWith(string)
+            if item:
+                break
+            
+        return item
+        
+    def __lt__(self, other):
+        if self.childCount() and not other.childCount():
+            return True
+        
+        if other.childCount() and not self.childCount():
+            return False
+        
+        return bool(self.text(0).lower() < other.text(0).lower())
+
+class SessionFolder:
+    name = ""
+    has_session = True
+    path = ""
+    
+    def __init__(self, name):
+        self.name = name
+        self.subfolders = []
+    
+    def setPath(self, path):
+        self.path = path
+    
+    def makeItem(self):
+        item = SessionItem([self.name])
+        
+        item.setData(0, Qt.UserRole, self.path)
+        if self.subfolders:
+            item.setIcon(0, QIcon.fromTheme('folder'))
+        
+        if not self.path:
+            item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
+        
+        for folder in self.subfolders:
+            sub_item = folder.makeItem()
+            item.addChild(sub_item)
+            
+        return item
+    
 class OpenSessionDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
         self.ui = ui_open_session.Ui_DialogOpenSession()
         self.ui.setupUi(self)
 
-        self.f_last_session_item = None
-
         self.ui.toolButtonFolder.clicked.connect(self.changeRootFolder)
-        self.ui.sessionList.currentItemChanged.connect(self.currentItemChanged)
+        self.ui.sessionList.currentItemChanged.connect(
+            self.currentItemChanged)
         self.ui.sessionList.setFocus(Qt.OtherFocusReason)
+        self.ui.sessionList.itemDoubleClicked.connect(self.goIfAny)
+        self.ui.sessionList.itemClicked.connect(self.deployItem)
         self.ui.filterBar.textEdited.connect(self.updateFilteredList)
         self.ui.filterBar.updownpressed.connect(self.updownPressed)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
+        self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
         
-        #session_root = RS.settings.value('default_session_root', '')
-        
-        self.ui.currentNsmFolder.setText(CommandLineArgs.session_root)
-
         self._signaler.add_sessions_to_list.connect(self.addSessions)
         self._signaler.root_changed.connect(self.rootChanged)
 
@@ -131,13 +203,19 @@ class OpenSessionDialog(ChildDialog):
 
         if self.daemon_launched_before:
             self.ui.toolButtonFolder.setVisible(False)
-            self.ui.currentNsmFolder.setVisible(False)
-            self.ui.labelNsmFolder.setVisible(False)
+            self.ui.currentSessionsFolder.setVisible(False)
+            self.ui.labelSessionsFolder.setVisible(False)
 
         self.server_will_accept = False
         self.has_selection = False
 
         self.serverStatusChanged(self._session.server_status)
+        
+        self.folders = []
+        self.all_items = []
+        
+        self._last_mouse_click = 0
+        
 
     def serverStatusChanged(self, server_status):
         self.ui.toolButtonFolder.setEnabled(
@@ -152,51 +230,71 @@ class OpenSessionDialog(ChildDialog):
         self.preventOk()
         
     def rootChanged(self, session_root):
-        self.ui.currentNsmFolder.setText(session_root)
+        self.ui.currentSessionsFolder.setText(session_root)
         self.ui.sessionList.clear()
+        self.folders.clear()
         self.toDaemon('/ray/server/list_sessions', 0)
 
     def addSessions(self, session_names):
         for session_name in session_names:
-            if session_name == RS.settings.value('last_session', type=str):
-                self.f_last_session_item = QListWidgetItem(session_name)
-                self.ui.sessionList.addItem(self.f_last_session_item)
-                self.ui.sessionList.setCurrentItem(self.f_last_session_item)
-            else:
-                self.ui.sessionList.addItem(session_name)
-
-            self.ui.sessionList.sortItems()
-
-            if self.f_last_session_item:
-                current_index = self.ui.sessionList.currentIndex()
-                self.ui.sessionList.scrollTo(current_index)
-            else:
-                self.ui.sessionList.setCurrentRow(0)
+            folder_div = session_name.split('/')
+            folders = self.folders
+            
+            for i in range(len(folder_div)):
+                f = folder_div[i]
+                for g in folders:
+                    if g.name == f:
+                        if i+1 == len(folder_div):
+                            g.setPath(session_name)
+                            
+                        folders = g.subfolders
+                        break
+                else:
+                    new_folder = SessionFolder(f)
+                    if i+1 == len(folder_div):
+                        new_folder.setPath(session_name)
+                    folders.append(new_folder)
+                    folders = new_folder.subfolders
+                        
+        self.ui.sessionList.clear()
+        
+        for folder in self.folders:
+            item = folder.makeItem()
+            self.ui.sessionList.addTopLevelItem(item)
+        
+        self.ui.sessionList.sortByColumn(0, Qt.AscendingOrder)
+        
+        # Try to select last used session
+        root_item = self.ui.sessionList.invisibleRootItem()
+        for i in range(root_item.childCount()):
+            item = root_item.child(i)
+            last_session_item = item.findItemWith(
+                RS.settings.value('last_session', type=str))
+            
+            if last_session_item:
+                self.ui.sessionList.setCurrentItem(last_session_item)
+                self.ui.sessionList.scrollToItem(last_session_item)
+                break
 
     def updateFilteredList(self, filt):
         filter_text = self.ui.filterBar.displayText()
+        root_item = self.ui.sessionList.invisibleRootItem()
 
-        # show all items
-        for i in range(self.ui.sessionList.count()):
-            self.ui.sessionList.item(i).setHidden(False)
-
-        liist = self.ui.sessionList.findItems(filter_text, Qt.MatchContains)
-
-        # hide all non matching items
-        for i in range(self.ui.sessionList.count()):
-            if self.ui.sessionList.item(i) not in liist:
-                self.ui.sessionList.item(i).setHidden(True)
+        ## hide all non matching items
+        for i in range(root_item.childCount()):
+            root_item.child(i).showConditionnaly(filter_text)
 
         # if selected item not in list, then select the first visible
-        if not self.ui.sessionList.currentItem(
-        ) or self.ui.sessionList.currentItem().isHidden():
-            for i in range(self.ui.sessionList.count()):
-                if not self.ui.sessionList.item(i).isHidden():
-                    self.ui.sessionList.setCurrentRow(i)
+        if (not self.ui.sessionList.currentItem()
+                or self.ui.sessionList.currentItem().isHidden()):
+            for i in range(root_item.childCount()):
+                item = root_item.child(i)
+                if not item.isHidden():
+                    self.ui.sessionList.setCurrentItem(item)
                     break
 
-        if not self.ui.sessionList.currentItem(
-        ) or self.ui.sessionList.currentItem().isHidden():
+        if (not self.ui.sessionList.currentItem()
+                or self.ui.sessionList.currentItem().isHidden()):
             self.ui.filterBar.setStyleSheet(
                 "QLineEdit { background-color: red}")
             self.ui.sessionList.setCurrentItem(None)
@@ -205,44 +303,57 @@ class OpenSessionDialog(ChildDialog):
             self.ui.sessionList.scrollTo(self.ui.sessionList.currentIndex())
 
     def updownPressed(self, key):
-        row = self.ui.sessionList.currentRow()
+        root_item = self.ui.sessionList.invisibleRootItem()
+        row = self.ui.sessionList.currentIndex().row()
+        
         if key == Qt.Key_Up:
             if row == 0:
                 return
             row -= 1
-            while self.ui.sessionList.item(row).isHidden():
+            while root_item.child(row).isHidden():
                 if row == 0:
                     return
                 row -= 1
         elif key == Qt.Key_Down:
-            if row == self.ui.sessionList.count() - 1:
+            if row == root_item.childCount() - 1:
                 return
             row += 1
-            while self.ui.sessionList.item(row).isHidden():
-                if row == self.ui.sessionList.count() - 1:
+            while root_item.child(row).isHidden():
+                if row == root_item.childCount() - 1:
                     return
                 row += 1
-        self.ui.sessionList.setCurrentRow(row)
+                
+        self.ui.sessionList.setCurrentItem(root_item.child(row))
 
     def currentItemChanged(self, item, previous_item):
-        self.has_selection = bool(item)
+        self.has_selection = bool(item and item.data(0, Qt.UserRole))
         self.preventOk()
 
     def preventOk(self):
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
             bool(self.server_will_accept and self.has_selection))
 
-    #def changeRootFolder(self):
-        #changeRootFolder(self)
-        #self.ui.currentNsmFolder.setText(default_session_root)
-        #self.ui.sessionList.clear()
-        ## self._server.startListSession()
-        #self.toDaemon('/ray/server/list_sessions', 0)
-
     def getSelectedSession(self):
         if self.ui.sessionList.currentItem():
-            return self.ui.sessionList.currentItem().text()
-
+            return self.ui.sessionList.currentItem().data(0, Qt.UserRole)
+    
+    def deployItem(self, item, column):
+        if not item.childCount():
+            return 
+        
+        if time.time() - self._last_mouse_click > 0.35:
+            item.setExpanded(not item.isExpanded())
+            
+        self._last_mouse_click = time.time()
+        
+    def goIfAny(self, item, column):
+        if item.childCount():
+            return 
+        
+        if (self.server_will_accept and self.has_selection
+            and self.ui.sessionList.currentItem().data(0, Qt.UserRole)):
+                self.accept()
+            
 
 class NewSessionDialog(ChildDialog):
     def __init__(self, parent, duplicate_window=False):
@@ -252,7 +363,7 @@ class NewSessionDialog(ChildDialog):
 
         self.is_duplicate = bool(duplicate_window)
 
-        self.ui.currentNsmFolder.setText(CommandLineArgs.session_root)
+        self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
         self.ui.toolButtonFolder.clicked.connect(self.changeRootFolder)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
         self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
@@ -260,15 +371,15 @@ class NewSessionDialog(ChildDialog):
 
         self.session_list = []
         self.template_list = []
+        self.sub_folders = []
 
         self._signaler.server_status_changed.connect(self.serverStatusChanged)
-
         self._signaler.add_sessions_to_list.connect(self.addSessionsToList)
+        self._signaler.session_template_found.connect(self.addTemplatesToList)
+        self._signaler.root_changed.connect(self.rootChanged)
         
         self.toDaemon('/ray/server/list_sessions', 1)
-
-        self._signaler.session_template_found.connect(self.addTemplatesToList)
-
+        
         if self.is_duplicate:
             self.ui.labelTemplate.setVisible(False)
             self.ui.comboBoxTemplate.setVisible(False)
@@ -280,11 +391,14 @@ class NewSessionDialog(ChildDialog):
 
         if self.daemon_launched_before:
             self.ui.toolButtonFolder.setVisible(False)
-            self.ui.currentNsmFolder.setVisible(False)
-            self.ui.labelNsmFolder.setVisible(False)
+            self.ui.currentSessionsFolder.setVisible(False)
+            self.ui.labelSessionsFolder.setVisible(False)
 
-        self.initComboBox()
+        self.initTemplatesComboBox()
         self.setLastTemplateSelected()
+        
+        self.ui.labelSubFolder.setVisible(False)
+        self.ui.comboBoxSubFolder.setVisible(False)
 
         self.server_will_accept = False
         self.text_is_valid = False
@@ -306,11 +420,13 @@ class NewSessionDialog(ChildDialog):
         self.preventOk()
     
     def rootChanged(self, session_root):
-        self.ui.currentNsmFolder.setText(session_root)
-        self.ui.sessionList.clear()
+        self.ui.currentSessionsFolder.setText(session_root)
+        self.session_list.clear()
+        self.sub_folders.clear()
+        self.initSubFolderCombobox()
         self.toDaemon('/ray/server/list_sessions', 1)
     
-    def initComboBox(self):
+    def initTemplatesComboBox(self):
         self.ui.comboBoxTemplate.clear()
         self.ui.comboBoxTemplate.addItem(
             _translate('session_template', "empty"))
@@ -320,6 +436,17 @@ class NewSessionDialog(ChildDialog):
                 "with JACK patch memory"))
         self.ui.comboBoxTemplate.insertSeparator(2)
 
+    def initSubFolderCombobox(self):
+        self.ui.comboBoxSubFolder.clear()
+        self.ui.comboBoxSubFolder.addItem(_translate('new_session', 'none'))
+        
+        for sub_folder in self.sub_folders:
+            self.ui.comboBoxSubFolder.addItem(QIcon.fromTheme('folder'), 
+                                              sub_folder)
+        
+        self.ui.labelSubFolder.setVisible(bool(self.sub_folders))
+        self.ui.comboBoxSubFolder.setVisible(bool(self.sub_folders))
+        
     def setLastTemplateSelected(self):
         last_used_template = RS.settings.value('last_used_template', type=str)
 
@@ -333,8 +460,26 @@ class NewSessionDialog(ChildDialog):
         if not last_used_template:
             self.ui.comboBoxTemplate.setCurrentIndex(1)
 
+    def setLastSubFolderSelected(self):
+        last_subfolder = RS.settings.value('last_subfolder', type=str)
+        
+        if last_subfolder:
+            self.ui.comboBoxSubFolder.setCurrentText(last_subfolder)
+        else:
+            self.ui.comboBoxSubFolder.setCurrentIndex(0)
+    
     def addSessionsToList(self, session_names):
         self.session_list += session_names
+        
+        for session_name in session_names:
+            if '/' in session_name:
+                new_dir = os.path.dirname(session_name)
+                if not new_dir in self.sub_folders:
+                    self.sub_folders.append(new_dir)
+        
+        self.sub_folders.sort()
+        self.initSubFolderCombobox()
+        self.setLastSubFolderSelected()
 
     def addTemplatesToList(self, template_list):
         for template in template_list:
@@ -346,7 +491,7 @@ class NewSessionDialog(ChildDialog):
 
         self.template_list.sort()
 
-        self.initComboBox()
+        self.initTemplatesComboBox()
 
         for template_name in self.template_list:
             self.ui.comboBoxTemplate.addItem(template_name)
@@ -354,7 +499,11 @@ class NewSessionDialog(ChildDialog):
         self.setLastTemplateSelected()
 
     def getSessionName(self):
-        return self.ui.lineEdit.text()
+        if self.ui.comboBoxSubFolder.currentIndex() > 0:
+            return '%s/%s' % (self.ui.comboBoxSubFolder.currentText(),
+                              self.ui.lineEdit.text())
+        else:
+            return self.ui.lineEdit.text()
 
     def getTemplateName(self):
         if self.ui.comboBoxTemplate.currentIndex() == 0:
@@ -364,17 +513,25 @@ class NewSessionDialog(ChildDialog):
             return '///withJACKPATCH'
 
         return self.ui.comboBoxTemplate.currentText()
-
+    
+    def getSubFolder(self):
+        if self.ui.comboBoxSubFolder.currentIndex() == 0:
+            return ""
+        
+        return  self.ui.comboBoxSubFolder.currentText()
+    
     def textChanged(self, text):
-        self.text_is_valid = bool(text and text not in self.session_list)
+        full_session_text = text
+        if self.ui.comboBoxSubFolder.currentIndex():
+            full_session_text = "%s/%s" % (
+                                    self.ui.comboBoxSubFolder.currentText(),
+                                    text)
+            
+        self.text_is_valid = bool(text
+                                  and full_session_text 
+                                        not in self.session_list)
         self.preventOk()
 
-    def changeRootFolder(self):
-        self.ui.currentNsmFolder.setText(default_session_root)
-        self.session_list.clear()
-        # self._server.startListSession()
-        self.toDaemon('/ray/server/list_sessions', 0)
-
     def preventOk(self):
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
             bool(self.server_will_accept and self.text_is_valid))
@@ -517,6 +674,8 @@ class ClientPropertiesDialog(ChildDialog):
         self.ui.checkBoxSaveStop.setChecked(self.client.check_last_save)
         self.ui.toolButtonIcon.setIcon(
             ray.getAppIcon(self.client.icon_name, self))
+        self.ui.lineEditIgnoredExtensions.setText(
+            self.client.ignored_extensions)
 
     def changeIconwithText(self, text):
         self.ui.toolButtonIcon.setIcon(ray.getAppIcon(text, self))
@@ -534,6 +693,7 @@ class ClientPropertiesDialog(ChildDialog):
         self.client.label = self.ui.lineEditLabel.text()
         self.client.icon_name = self.ui.lineEditIcon.text()
         self.client.check_last_save = self.ui.checkBoxSaveStop.isChecked()
+        self.client.ignored_extensions = self.ui.lineEditIgnoredExtensions.text()
         self.client.sendPropertiesToDaemon()
         # better for user to wait a little before close the window
         QTimer.singleShot(150, self.accept)
@@ -558,7 +718,11 @@ class ClientTrashDialog(ChildDialog):
         self.ui.pushButtonRemove.clicked.connect(self.removeClient)
 
     def serverStatusChanged(self, server_status):
-        if server_status in (ray.ServerStatus.CLOSE, ray.ServerStatus.OFF):
+        if server_status in (ray.ServerStatus.CLOSE,
+                             ray.ServerStatus.OFF,
+                             ray.ServerStatus.OUT_SAVE,
+                             ray.ServerStatus.OUT_SNAPSHOT,
+                             ray.ServerStatus.WAIT_USER):
             self.reject()
 
     def removeClient(self):
@@ -687,230 +851,38 @@ class AboutRaySessionDialog(ChildDialog):
         self.ui.labelRayAndVersion.setText(all_text % ray.VERSION)
 
 
-class AddApplicationDialog(ChildDialog):
-    def __init__(self, parent):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui_add_application.Ui_DialogAddApplication()
-        self.ui.setupUi(self)
-
-        self.ui.checkBoxFactory.setChecked(RS.settings.value(
-            'AddApplication/factory_box', True, type=bool))
-        self.ui.checkBoxUser.setChecked(RS.settings.value(
-            'AddApplication/user_box', True, type=bool))
-
-        self.ui.checkBoxFactory.stateChanged.connect(self.factoryBoxChanged)
-        self.ui.checkBoxUser.stateChanged.connect(self.userBoxChanged)
-
-        self.ui.templateList.currentItemChanged.connect(
-            self.currentItemChanged)
-        self.ui.templateList.setFocus(Qt.OtherFocusReason)
-        self.ui.filterBar.textEdited.connect(self.updateFilteredList)
-        self.ui.filterBar.updownpressed.connect(self.updownPressed)
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
-
-        self._signaler.user_client_template_found.connect(
-            self.addUserTemplates)
-        self._signaler.factory_client_template_found.connect(
-            self.addFactoryTemplates)
-        self.toDaemon('/ray/server/list_user_client_templates')
-        self.toDaemon('/ray/server/list_factory_client_templates')
-
-        self.user_template_list = []
-        self.factory_template_list = []
-
-        self.server_will_accept = False
-        self.has_selection = False
-
-        self.serverStatusChanged(self._session.server_status)
-
-    def factoryBoxChanged(self, state):
-        if not state:
-            self.ui.checkBoxUser.setChecked(True)
-
-        self.updateFilteredList()
-
-    def userBoxChanged(self, state):
-        if not state:
-            self.ui.checkBoxFactory.setChecked(True)
-
-        self.updateFilteredList()
-
-    def serverStatusChanged(self, server_status):
-        self.server_will_accept = bool(
-            server_status not in (
-                ray.ServerStatus.OFF,
-                ray.ServerStatus.CLOSE) and not self.server_copying)
-        self.preventOk()
-
-    def addUserTemplates(self, template_list):
-        for template in template_list:
-            template_name = template
-            icon_name = ''
-
-            if '/' in template:
-                template_name = template.split('/')[0]
-                icon_name = template.split('/')[1]
-
-            self.user_template_list.append(template_name)
-
-            self.ui.templateList.addItem(
-                QListWidgetItem(
-                    ray.getAppIcon(
-                        icon_name,
-                        self),
-                    template_name,
-                    self.ui.templateList))
-
-            self.ui.templateList.sortItems()
-
-        self.updateFilteredList()
-
-    def addFactoryTemplates(self, template_list):
-        for template in template_list:
-            template_name = template
-            icon_name = ''
-
-            if '/' in template:
-                template_name = template.split('/')[0]
-                icon_name = template.split('/')[1]
-
-            self.factory_template_list.append(template_name)
-
-            self.ui.templateList.addItem(
-                QListWidgetItem(
-                    ray.getAppIcon(
-                        icon_name,
-                        self),
-                    template_name,
-                    self.ui.templateList))
-
-            self.ui.templateList.sortItems()
-
-        self.updateFilteredList()
-
-    def updateFilteredList(self, filt=''):
-        filter_text = self.ui.filterBar.displayText()
-
-        # show all items
-        for i in range(self.ui.templateList.count()):
-            self.ui.templateList.item(i).setHidden(False)
-
-        liist = self.ui.templateList.findItems(filter_text, Qt.MatchContains)
-
-        seen_template_list = []
-
-        # hide all non matching items
-        for i in range(self.ui.templateList.count()):
-            template_name = self.ui.templateList.item(i).text()
-
-            if self.ui.templateList.item(i) not in liist:
-                self.ui.templateList.item(i).setHidden(True)
-                continue
-
-            if self.ui.checkBoxFactory.isChecked() and self.ui.checkBoxUser.isChecked():
-                if template_name in seen_template_list:
-                    self.ui.templateList.item(i).setHidden(True)
-                else:
-                    seen_template_list.append(template_name)
-
-            elif self.ui.checkBoxFactory.isChecked():
-                if template_name not in self.factory_template_list:
-                    self.ui.templateList.item(i).setHidden(True)
-                    continue
-
-                if template_name in seen_template_list:
-                    self.ui.templateList.item(i).setHidden(True)
-                else:
-                    seen_template_list.append(template_name)
-
-            elif self.ui.checkBoxUser.isChecked():
-                if template_name not in self.user_template_list:
-                    self.ui.templateList.item(i).setHidden(True)
-
-                if template_name in seen_template_list:
-                    self.ui.templateList.item(i).setHidden(True)
-                else:
-                    seen_template_list.append(template_name)
-
-        # if selected item not in list, then select the first visible
-        if not self.ui.templateList.currentItem(
-        ) or self.ui.templateList.currentItem().isHidden():
-            for i in range(self.ui.templateList.count()):
-                if not self.ui.templateList.item(i).isHidden():
-                    self.ui.templateList.setCurrentRow(i)
-                    break
-
-        if not self.ui.templateList.currentItem(
-        ) or self.ui.templateList.currentItem().isHidden():
-            self.ui.filterBar.setStyleSheet(
-                "QLineEdit { background-color: red}")
-            self.ui.templateList.setCurrentItem(None)
-        else:
-            self.ui.filterBar.setStyleSheet("")
-            self.ui.templateList.scrollTo(self.ui.templateList.currentIndex())
-
-    def updownPressed(self, key):
-        row = self.ui.templateList.currentRow()
-        if key == Qt.Key_Up:
-            if row == 0:
-                return
-            row -= 1
-            while self.ui.templateList.item(row).isHidden():
-                if row == 0:
-                    return
-                row -= 1
-        elif key == Qt.Key_Down:
-            if row == self.ui.templateList.count() - 1:
-                return
-            row += 1
-            while self.ui.templateList.item(row).isHidden():
-                if row == self.ui.templateList.count() - 1:
-                    return
-                row += 1
-        self.ui.templateList.setCurrentRow(row)
-
-    def currentItemChanged(self, item, previous_item):
-        self.has_selection = bool(item)
-        self.preventOk()
-
-    def preventOk(self):
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
-            bool(self.server_will_accept and self.has_selection))
-
-    def getSelectedTemplate(self):
-        if self.ui.templateList.currentItem():
-            return self.ui.templateList.currentItem().text()
-
-    def isTemplateFactory(self, template_name):
-        if not self.ui.checkBoxUser.isChecked():
-            return True
-
-        # If both factory and user boxes are checked, priority to user template
-        if template_name in self.user_template_list:
-            return False
-
-        return True
-
-    def saveCheckBoxes(self):
-        RS.settings.setValue(
-            'AddApplication/factory_box',
-            self.ui.checkBoxFactory.isChecked())
-        RS.settings.setValue(
-            'AddApplication/user_box',
-            self.ui.checkBoxUser.isChecked())
-        RS.settings.sync()
-
-
 class NewExecutableDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
         self.ui = ui_new_executable.Ui_DialogNewExecutable()
         self.ui.setupUi(self)
+        
+        self.ui.groupBoxAdvanced.setVisible(False)
+        self.resize(0, 0)
+        self.ui.labelPrefixMode.setToolTip(
+            self.ui.comboBoxPrefixMode.toolTip())
+        self.ui.labelClientId.setToolTip(self.ui.lineEditClientId.toolTip())
+        
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
         self.ui.lineEdit.setFocus(Qt.OtherFocusReason)
         self.ui.lineEdit.textChanged.connect(self.textChanged)
 
         self.ui.checkBoxProxy.stateChanged.connect(self.proxyStateChanged)
+        
+        self.ui.lineEditPrefix.setEnabled(False)
+        self.ui.toolButtonAdvanced.clicked.connect(self.showAdvanced)
+        
+        self.ui.comboBoxPrefixMode.addItem(
+            _translate('new_executable', 'Custom'))
+        self.ui.comboBoxPrefixMode.addItem(
+            _translate('new_executable', 'Client Name'))
+        self.ui.comboBoxPrefixMode.addItem(
+            _translate('new_executable', 'Session Name'))
+        self.ui.comboBoxPrefixMode.setCurrentIndex(2)
+        
+        self.ui.comboBoxPrefixMode.currentIndexChanged.connect(
+            self.prefixModeChanged)
+        
         self._signaler.new_executable.connect(self.addExecutableToCompleter)
         self.toDaemon('/ray/server/list_path')
 
@@ -920,11 +892,18 @@ class NewExecutableDialog(ChildDialog):
         self.ui.lineEdit.setCompleter(self.completer)
 
         self.ui.lineEdit.returnPressed.connect(self.closeNow)
-
+        
         self.serverStatusChanged(self._session.server_status)
-
+        
         self.text_will_accept = False
 
+    def showAdvanced(self):
+        self.ui.groupBoxAdvanced.setVisible(True)
+        self.ui.toolButtonAdvanced.setVisible(False)
+    
+    def prefixModeChanged(self, index):
+        self.ui.lineEditPrefix.setEnabled(bool(index == 0))
+    
     def addExecutableToCompleter(self, executable_list):
         self.exec_list += executable_list
         self.exec_list.sort()
@@ -938,7 +917,14 @@ class NewExecutableDialog(ChildDialog):
 
     def runViaProxy(self):
         return bool(self.ui.checkBoxProxy.isChecked())
-
+    
+    def getSelection(self):
+        return (self.ui.lineEdit.text(),
+                self.ui.checkBoxProxy.isChecked(),
+                self.ui.comboBoxPrefixMode.currentIndex(),
+                self.ui.lineEditPrefix.text(),
+                self.ui.lineEditClientId.text())
+    
     def proxyStateChanged(self, state):
         self.ui.buttonBox.button(
             QDialogButtonBox.Ok).setEnabled(
@@ -950,11 +936,16 @@ class NewExecutableDialog(ChildDialog):
             self.ui.checkBoxProxy.isChecked() or self.text_will_accept)
 
     def closeNow(self):
-        if self.ui.lineEdit.text() in self.exec_list or self.ui.checkBoxProxy.isChecked():
+        if (self.ui.lineEdit.text() in self.exec_list
+                or self.ui.checkBoxProxy.isChecked()):
             self.accept()
 
     def serverStatusChanged(self, server_status):
-        if server_status in (ray.ServerStatus.CLOSE, ray.ServerStatus.OFF):
+        if server_status in (ray.ServerStatus.OUT_SAVE,
+                             ray.ServerStatus.OUT_SNAPSHOT,
+                             ray.ServerStatus.WAIT_USER,
+                             ray.ServerStatus.CLOSE,
+                             ray.ServerStatus.OFF):
             self.reject()
 
 
@@ -980,26 +971,23 @@ class StopClientDialog(ChildDialog):
                                                                                                                                  minutes)
 
             self.ui.label.setText(text)
+            
+            self.client.status_changed.connect(self.serverUpdatesClientStatus)
 
         self.ui.pushButtonSaveStop.clicked.connect(self.saveAndStop)
         self.ui.checkBox.stateChanged.connect(self.checkBoxClicked)
 
-        self._signaler.client_status_changed.connect(
-            self.serverUpdatesClientStatus)
+        
 
     def saveAndStop(self):
         self.wait_for_save = True
-        # self._server.saveClient(self.client_id)
         self.toDaemon('/ray/client/save', self.client_id)
 
     def checkBoxClicked(self, state):
         self.client.check_last_save = not bool(state)
         self.client.sendPropertiesToDaemon()
 
-    def serverUpdatesClientStatus(self, client_id, status):
-        if client_id != self.client_id:
-            return
-
+    def serverUpdatesClientStatus(self, status):
         if status in (ray.ClientStatus.STOPPED, ray.ClientStatus.REMOVED):
             self.reject()
             return
@@ -1009,6 +997,47 @@ class StopClientDialog(ChildDialog):
             self.accept()
 
 
+class StopClientNoSaveDialog(ChildDialog):
+    def __init__(self, parent, client_id):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_stop_client_no_save.Ui_Dialog()
+        self.ui.setupUi(self)
+        
+        self.client_id = client_id
+        self.client = self._session.getClient(client_id)
+
+        if self.client:
+            text = self.ui.label.text() % self.client.prettierName()
+            self.ui.label.setText(text)
+            self.client.status_changed.connect(self.serverUpdatesClientStatus)
+        
+        self.ui.checkBox.stateChanged.connect(self.checkBoxClicked)
+        self.ui.pushButtonCancel.setFocus(True)
+        
+    def serverUpdatesClientStatus(self, status):
+        if status in (ray.ClientStatus.STOPPED, ray.ClientStatus.REMOVED):
+            self.reject()
+            return
+        
+    def checkBoxClicked(self, state):
+        self.client.check_last_save = not bool(state)
+        self.client.sendPropertiesToDaemon()
+
+
+class SnapShotProgressDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_snapshot_progress.Ui_Dialog()
+        self.ui.setupUi(self)
+        self._signaler.server_progress.connect(self.serverProgress)
+        
+    def serverStatusChanged(self, server_status):
+        self.close()
+        
+    def serverProgress(self, value):
+        self.ui.progressBar.setValue(value * 100)
+        
+
 class DaemonUrlWindow(ChildDialog):
     def __init__(self, parent, err_code, ex_url):
         ChildDialog.__init__(self, parent)
@@ -1037,7 +1066,7 @@ class DaemonUrlWindow(ChildDialog):
         elif err_code == ErrDaemon.WRONG_VERSION:
             error_text = _translate(
                 "url_window",
-                "<p>daemon at<br><strong>%s</strong><br>uses an other Ray Session version.<.p>") % ex_url
+                "<p>daemon at<br><strong>%s</strong><br>uses another %s version.<.p>") % (ex_url, ray.APP_TITLE)
         else:
             error_text = _translate("url window", "<p align=\"left\">To run a network session,<br>open a terminal on another computer of this network.<br>Launch ray-daemon on port 1234 (for example)<br>by typing the command :</p><p align=\"left\"><code>ray-daemon -p 1234</code></p><p align=\"left\">Then paste below the first url<br>that ray-daemon gives you at startup.</p><p></p>")
 
@@ -1078,10 +1107,58 @@ class DaemonUrlWindow(ChildDialog):
         return self.ui.lineEdit.text()
 
 
+class WaitingCloseUserDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_waiting_close_user.Ui_Dialog()
+        self.ui.setupUi(self)
+        
+        if isDarkTheme(self):
+            self.ui.labelSaveIcon.setPixmap(
+                QPixmap(':scalable/breeze-dark/document-nosave.svg'))
+        
+        self.ui.pushButtonOk.setFocus(True)
+        self.ui.pushButtonUndo.clicked.connect(self.undoClose)
+        self.ui.pushButtonSkip.clicked.connect(self.skip)
+        self.ui.checkBox.setChecked(
+            bool(RS.settings.value(
+                'hide_wait_close_user_dialog', False, type=bool)))
+        
+        self.ui.checkBox.clicked.connect(self.checkBoxClicked)
+        
+    def serverStatusChanged(self, server_status):
+        if server_status != ray.ServerStatus.WAIT_USER:
+            self.accept()
+    
+    def undoClose(self):
+        self.toDaemon('/ray/session/cancel_close')
+    
+    def skip(self):
+        self.toDaemon('/ray/session/skip_wait_user')
+        
+    def checkBoxClicked(self, state):
+        RS.settings.setValue('hide_wait_close_user_dialog',
+                             bool(state), type=bool)
+
+class DonationsDialog(ChildDialog):
+    def __init__(self, parent, display_no_again):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_donations.Ui_Dialog()
+        self.ui.setupUi(self)
+        
+        print('display_no_again', display_no_again)
+        self.ui.checkBox.setVisible(display_no_again)
+        self.ui.checkBox.clicked.connect(self.checkBoxClicked)
+        
+    def checkBoxClicked(self, state):
+        RS.settings.setValue('hide_donations', state)
+        
+
 class ErrorDialog(ChildDialog):
-    def __init__(self, parent, osc_args):
+    def __init__(self, parent, message):
         ChildDialog.__init__(self, parent)
         self.ui = ui_error_dialog.Ui_Dialog()
         self.ui.setupUi(self)
 
-        self.ui.label.setText(osc_args[2])
+        self.ui.label.setText(message)
+
