@@ -1,12 +1,18 @@
 import time
 import sys
+from PyQt5.QtCore import QObject, pyqtSignal
+
 import child_dialogs
+import snapshots_dialog
 import ray
 from gui_server_thread import GUIServerThread
 
 
-class Client(object):
+class Client(QObject):
+    status_changed = pyqtSignal(int)
+    
     def __init__(self, session, client_data, trashed=False):
+        QObject.__init__(self)
         self._session = session
         self._main_win = self._session._main_win
 
@@ -15,11 +21,12 @@ class Client(object):
         self.arguments       = client_data.arguments
         self.name            = client_data.name
         self.prefix_mode     = client_data.prefix_mode
-        self.project_path    = client_data.project_path
+        self.custom_prefix   = client_data.custom_prefix
         self.label           = client_data.label
         self.icon_name       = client_data.icon
         self.capabilities    = client_data.capabilities
         self.check_last_save = client_data.check_last_save
+        self.ignored_extensions = client_data.ignored_extensions
 
         self.status = ray.ClientStatus.STOPPED
         self.previous_status = ray.ClientStatus.STOPPED
@@ -27,6 +34,7 @@ class Client(object):
         self.gui_visible = False
         self.has_dirty = False
         self.dirty_state = True
+        self.no_save_level = 0
         self.last_save = time.time()
 
         self.widget = self._main_win.createClientWidget(self)
@@ -36,6 +44,7 @@ class Client(object):
     def setStatus(self, status):
         self.previous_status = self.status
         self.status = status
+        self.status_changed.emit(status)
 
         if (not self.has_dirty
             and self.status == ray.ClientStatus.READY
@@ -58,6 +67,10 @@ class Client(object):
         self.dirty_state = bool_dirty
         self.widget.setDirtyState(bool_dirty)
 
+    def setNoSaveLevel(self, no_save_level):
+        self.no_save_level = no_save_level
+        self.widget.setNoSaveLevel(no_save_level)
+    
     def setProgress(self, progress):
         self.widget.setProgress(progress)
 
@@ -77,7 +90,7 @@ class Client(object):
         self.arguments       = client_data.arguments
         self.name            = client_data.name
         self.prefix_mode     = client_data.prefix_mode
-        self.project_path    = client_data.project_path
+        self.custom_prefix   = client_data.custom_prefix
         self.label           = client_data.label
         self.icon_name       = client_data.icon
         self.capabilities    = client_data.capabilities
@@ -108,11 +121,12 @@ class Client(object):
                         self.arguments,
                         self.name,
                         self.prefix_mode,
-                        self.project_path,
+                        self.custom_prefix,
                         self.label,
                         self.icon_name,
                         self.capabilities,
-                        int(self.check_last_save))
+                        int(self.check_last_save),
+                        self.ignored_extensions)
 
     def showPropertiesDialog(self):
         self.properties_dialog.updateContents()
@@ -128,7 +142,8 @@ class Client(object):
             self.setGuiEnabled()
 
     def hasBeenRecentlySaved(self):
-        if (time.time() - self.last_save) >= 60:  # last save more than 60 seconds ago
+        if (time.time() - self.last_save) >= 60:  
+            # last save more than 60 seconds ago
             return False
 
         return True
