@@ -1,24 +1,23 @@
 import time
-from PyQt5.QtWidgets import QApplication, QMainWindow, QMenu, QInputDialog, QBoxLayout, QListWidgetItem, QFrame, QDialog, QDialogButtonBox, QFileDialog, QMessageBox, QCompleter, QAction, QToolButton, QAbstractItemView, QLabel, QLineEdit
+from PyQt5.QtWidgets import (QApplication, QMainWindow, QMenu, QInputDialog,
+        QBoxLayout, QListWidgetItem, QFrame, QDialog, QDialogButtonBox,
+        QFileDialog, QMessageBox, QCompleter, QAction, QToolButton,
+        QAbstractItemView, QLabel, QLineEdit)
 from PyQt5.QtGui import QIcon, QCursor, QPalette, QPixmap, QFontDatabase
-from PyQt5.QtCore import QTimer, QProcess, pyqtSignal, pyqtSlot, QObject, QSize, Qt, QSettings, qDebug, QLocale, QTranslator
+from PyQt5.QtCore import (QTimer, QProcess, pyqtSignal, pyqtSlot, QObject,
+                          QSize, Qt, QSettings, qDebug, QLocale, QTranslator)
 
 from gui_tools import (RS, RayIcon, CommandLineArgs, _translate,
-                       serverStatusString)
-from child_dialogs import (
-    OpenSessionDialog, NewSessionDialog, SaveTemplateSessionDialog,
-    SaveTemplateClientDialog, EditExecutableDialog, ClientPropertiesDialog,
-    ClientTrashDialog, AbortSessionDialog, AbortServerCopyDialog,
-    AbortClientCopyDialog, OpenNsmSessionInfoDialog, QuitAppDialog,
-    AboutRaySessionDialog, AddApplicationDialog, NewExecutableDialog,
-    StopClientDialog, DaemonUrlWindow, ErrorDialog)
+                       serverStatusString, isDarkTheme)
+import add_application_dialog
+import child_dialogs
+import snapshots_dialog
 from gui_server_thread import GUIServerThread
 from gui_client import TrashedClient
 
 import ray
 import list_widget_clients
 
-
 import ui_raysession
 import ui_client_slot
 
@@ -47,6 +46,13 @@ class MainWindow(QMainWindow):
         self.timer_flicker_open.timeout.connect(self.flashOpen)
         self.flash_open_list = []
         self.flash_open_bool = False
+        
+        # timer for too long snapshots, display snapshot progress dialog
+        self.timer_snapshot = QTimer()
+        self.timer_snapshot.setSingleShot(True)
+        self.timer_snapshot.setInterval(2000)
+        self.timer_snapshot.timeout.connect(self.showSnapshotProgressDialog)
+        
 
         self.server_copying = False
 
@@ -75,6 +81,8 @@ class MainWindow(QMainWindow):
             self.ui.actionAddApplication)
         self.ui.toolButtonAddExecutable.setDefaultAction(
             self.ui.actionAddExecutable)
+        self.ui.toolButtonSnapshots.setDefaultAction(
+            self.ui.actionReturnToAPreviousState)
 
         # connect actions
         self.ui.actionNewSession.triggered.connect(self.createNewSession)
@@ -83,27 +91,36 @@ class MainWindow(QMainWindow):
         self.ui.actionSaveSession.triggered.connect(self.saveSession)
         self.ui.actionCloseSession.triggered.connect(self.closeSession)
         self.ui.actionAbortSession.triggered.connect(self.abortSession)
-        self.ui.actionRenameSession.triggered.connect(self.renameSessionAction)
-        self.ui.actionDuplicateSession.triggered.connect(self.duplicateSession)
+        self.ui.actionRenameSession.triggered.connect(
+            self.renameSessionAction)
+        self.ui.actionDuplicateSession.triggered.connect(
+            self.duplicateSession)
         self.ui.actionSaveTemplateSession.triggered.connect(
             self.saveTemplateSession)
-        self.ui.actionOpenSessionFolder.triggered.connect(self.openFileManager)
+        self.ui.actionReturnToAPreviousState.triggered.connect(
+            self.returnToAPreviousState)
+        self.ui.actionOpenSessionFolder.triggered.connect(
+            self.openFileManager)
         self.ui.actionAddApplication.triggered.connect(self.addApplication)
         self.ui.actionAddExecutable.triggered.connect(self.addExecutable)
         self.ui.actionKeepFocus.toggled.connect(self.toggleKeepFocus)
-        self.ui.actionSaveAllFromSavedClient.toggled.connect(
+        self.ui.actionSaveAllFromSavedClient.triggered.connect(
             self.saveAllFromClientToggled)
-        self.ui.actionBookmarkSessionFolder.toggled.connect(
+        self.ui.actionBookmarkSessionFolder.triggered.connect(
             self.bookmarkSessionFolderToggled)
-        self.ui.actionDesktopsMemory.toggled.connect(
+        self.ui.actionDesktopsMemory.triggered.connect(
             self.desktopsMemoryToggled)
+        self.ui.actionAutoSnapshot.triggered.connect(
+            self.autoSnapshotToggled)
         self.ui.actionAboutRaySession.triggered.connect(self.aboutRaySession)
         self.ui.actionAboutQt.triggered.connect(QApplication.aboutQt)
+        self.ui.actionDonate.triggered.connect(self.donate)
 
-        self.ui.lineEditServerStatus.copyAborted.connect(self.abortCopy)
+        self.ui.lineEditServerStatus.statusPressed.connect(
+            self.statusBarPressed)
         self.ui.stackedWidgetSessionName.name_changed.connect(
             self.renameSession)
-
+        
         # set control menu
         self.controlMenu = QMenu()
         self.controlMenu.addAction(self.ui.actionShowMenuBar)
@@ -113,6 +130,7 @@ class MainWindow(QMainWindow):
         self.controlMenu.addSeparator()
         self.controlMenu.addAction(self.ui.actionSaveAllFromSavedClient)
         self.controlMenu.addAction(self.ui.actionBookmarkSessionFolder)
+        self.controlMenu.addAction(self.ui.actionAutoSnapshot)
         self.controlMenu.addAction(self.ui.actionDesktopsMemory)
 
         self.controlToolButton = self.ui.toolBar.widgetForAction(
@@ -122,7 +140,14 @@ class MainWindow(QMainWindow):
 
         self.ui.toolButtonControl2.setPopupMode(QToolButton.InstantPopup)
         self.ui.toolButtonControl2.setMenu(self.controlMenu)
-
+        
+        # set favorites menu
+        self.favorites_menu = QMenu(_translate('menu', 'Favorites'))
+        self.favorites_menu.setIcon(QIcon(':scalable/breeze/star-yellow'))
+        self.ui.toolButtonFavorites.setPopupMode(QToolButton.InstantPopup)
+        self.ui.toolButtonFavorites.setMenu(self.favorites_menu)
+        self.ui.menuAdd.addMenu(self.favorites_menu)
+        
         # set trash menu
         self.trashMenu = QMenu()
         self.ui.trashButton.setPopupMode(QToolButton.InstantPopup)
@@ -130,46 +155,41 @@ class MainWindow(QMainWindow):
 
         # connect OSC signals from daemon
         sg = self._signaler
-
-        sg.new_client_added.connect(self.serverAddsClient)
-        sg.client_removed.connect(self.serverRemovesClient)
-        sg.client_status_changed.connect(self.serverUpdatesClientStatus)
-        sg.client_has_gui.connect(self.serverSetsClientHasGui)
-        sg.client_gui_visible_sig.connect(self.serverSetsClientGuiState)
-        sg.client_dirty_sig.connect(self.serverSetsClientDirtyState)
-        sg.client_switched.connect(self.serverSwitchesClient)
-        sg.client_progress.connect(self.serverClientProgress)
-        sg.client_still_running.connect(self.serverStillRunningClient)
-        sg.client_updated.connect(self.serverUpdatesClientProperties)
-        sg.new_message_sig.connect(self.serverPrintsMessage)
-        sg.session_name_sig.connect(self.serverRenamesSession)
-        sg.session_renameable.connect(self.serverSessionRenameable)
         sg.server_progress.connect(self.serverProgress)
         sg.server_status_changed.connect(self.serverChangeServerStatus)
         sg.server_copying.connect(self.serverCopying)
-        sg.error_message.connect(self.serverSendsError)
-        sg.opening_session.connect(self.serverOpensNsmSession)
-        sg.clients_reordered.connect(self.serverReorderClients)
-        sg.trash_add.connect(self.serverTrashAdd)
-        sg.trash_remove.connect(self.serverTrashRemove)
-        sg.trash_clear.connect(self.serverTrashClear)
-        sg.trash_dialog.connect(self.showClientTrashDialog)
         sg.daemon_url_request.connect(self.showDaemonUrlWindow)
-        sg.daemon_nsm_locked.connect(self.setNsmLocked)
-        sg.daemon_options.connect(self.setDaemonOptions)
-
-        # self.connectListWidgetRequests()
-
-        if self.ui.actionAddExecutable.icon().isNull():
-            self.ui.actionAddExecutable.setIcon(QIcon.fromTheme('system-run'))
+        
+        # set spare icons if system icons not avalaible
+        dark = isDarkTheme(self)
+        
+        if self.ui.actionNewSession.icon().isNull():
+            self.ui.actionNewSession.setIcon(RayIcon('folder-new', dark))
+        if self.ui.actionOpenSession.icon().isNull():
+            self.ui.actionOpenSession.setIcon(RayIcon('document-open', dark))
+            
         if self.ui.actionControlMenu.icon().isNull():
             self.ui.actionControlMenu.setIcon(
                 QIcon.fromTheme('configuration_section'))
-        if self.ui.trashButton.icon().isNull():
-            self.ui.trashButton.setIcon(QIcon.fromTheme('user-trash'))
-
-        dark = bool(QApplication.palette().brush(
-            2, QPalette.WindowText).color().lightness() > 128)
+            if self.ui.actionControlMenu.icon().isNull():
+                self.ui.actionControlMenu.setIcon(RayIcon('configure', dark))
+        
+        if self.ui.actionOpenSessionFolder.icon().isNull():
+            self.ui.actionOpenSessionFolder.setIcon(
+                                        RayIcon('system-file-manager', dark))
+        
+        if self.ui.actionAddApplication.icon().isNull():
+            self.ui.actionAddApplication.setIcon(RayIcon('list-add', dark))
+        
+        if self.ui.actionAddExecutable.icon().isNull():
+            self.ui.actionAddExecutable.setIcon(QIcon.fromTheme('system-run'))
+            if self.ui.actionAddExecutable.icon().isNull():
+                self.ui.actionAddExecutable.setIcon(RayIcon('run-install'))
+            
+        self.ui.actionReturnToAPreviousState.setIcon(
+                                        RayIcon('media-seek-backward', dark))
+        
+        self.ui.trashButton.setIcon(RayIcon('trash-empty', dark))
 
         self.ui.actionDuplicateSession.setIcon(
             RayIcon('xml-node-duplicate', dark))
@@ -188,6 +208,11 @@ class MainWindow(QMainWindow):
         if not self._daemon_manager.is_local:
             self.ui.actionKeepFocus.setChecked(False)
             self.ui.actionKeepFocus.setEnabled(False)
+        
+        self.server_progress = 0.0
+        self.progress_dialog_visible = False
+        
+        self.has_git = False
 
     def createClientWidget(self, client):
         return self.ui.listWidget.createClientWidget(client)
@@ -198,6 +223,7 @@ class MainWindow(QMainWindow):
         # (when reorder widgets sometimes one widget is totally hidden 
         # until user resize the window)
         # It has to be modified when ui_raysession is modified.
+
         self.ui.listWidget.clear()
         self.ui.verticalLayout.removeWidget(self.ui.listWidget)
         del self.ui.listWidget
@@ -211,10 +237,9 @@ class MainWindow(QMainWindow):
         self.ui.listWidget.setUniformItemSizes(True)
         self.ui.listWidget.setBatchSize(80)
         self.ui.listWidget.setObjectName("listWidget")
+        self.ui.listWidget.setSession(self._session)
         self.ui.verticalLayout.addWidget(self.ui.listWidget)
 
-        # self.connectListWidgetRequests()
-
     def setNsmLocked(self, nsm_locked):
         self.ui.actionNewSession.setEnabled(not nsm_locked)
         self.ui.actionOpenSession.setEnabled(not nsm_locked)
@@ -240,12 +265,24 @@ class MainWindow(QMainWindow):
             bool(options & ray.Option.BOOKMARK_SESSION))
         self.ui.actionDesktopsMemory.setChecked(
             bool(options & ray.Option.DESKTOPS_MEMORY))
+        self.ui.actionAutoSnapshot.setChecked(
+            bool(options & ray.Option.SNAPSHOTS))
 
         has_wmctrl = bool(options & ray.Option.HAS_WMCTRL)
         self.ui.actionDesktopsMemory.setEnabled(has_wmctrl)
         if has_wmctrl:
             self.ui.actionDesktopsMemory.setText(
                 _translate('actions', 'Desktops Memory'))
+            
+        has_git = bool(options & ray.Option.HAS_GIT)
+        self.ui.actionAutoSnapshot.setEnabled(has_git)
+        self.ui.actionReturnToAPreviousState.setVisible(has_git)
+        self.ui.toolButtonSnapshots.setVisible(has_git)
+        if has_git:
+            self.ui.actionAutoSnapshot.setText(
+                _translate('actions', 'Auto Snapshot at Save'))
+        
+        self.has_git = has_git
 
     def toDaemon(self, *args):
         server = GUIServerThread.instance()
@@ -278,6 +315,9 @@ class MainWindow(QMainWindow):
     def desktopsMemoryToggled(self, state):
         self.toDaemon('/ray/option/desktops_memory', int(state))
 
+    def autoSnapshotToggled(self, state):
+        self.toDaemon('/ray/option/snapshots', int(state))
+        
     def flashOpen(self):
         for client in self._session.client_list:
             if client.status == ray.ClientStatus.OPEN:
@@ -289,35 +329,20 @@ class MainWindow(QMainWindow):
         if (self._daemon_manager.launched_before
                 and not CommandLineArgs.under_nsm):
             self.quitAppNow()
-            return
+            return True
 
         if self._session.isRunning():
-            dialog = QuitAppDialog(self)
+            dialog = child_dialogs.QuitAppDialog(self)
             dialog.exec()
-            if dialog.result():
-                self.quitAppNow()
-        else:
-            self.quitAppNow()
+            if not dialog.result():
+                return False
+            
+        self.quitAppNow()
+        return True
 
     def quitAppNow(self):
         self._daemon_manager.stop()
 
-    def newClientTemplate(self, client_id):
-        dialog = SaveTemplateClientDialog(self)
-        dialog.exec()
-        if not dialog.result():
-            return
-
-        template_name = dialog.getTemplateName()
-        self.toDaemon('/ray/client/save_as_template', client_id, template_name)
-
-    def openClientProperties(self, client_id):
-        client = self._session.getClient(client_id)
-        if not client:
-            return
-
-        client.showPropertiesDialog()
-
     def updateClientLabel(self, client_id, label):
         client = self._session.getClient(client_id)
         if not client:
@@ -327,45 +352,48 @@ class MainWindow(QMainWindow):
 
     def createNewSession(self):
         self.ui.dockWidgetMessages.setVisible(False)
-        dialog = NewSessionDialog(self)
+        dialog = child_dialogs.NewSessionDialog(self)
         dialog.exec()
         if not dialog.result():
             return
 
         session_name = dialog.getSessionName()
         template_name = dialog.getTemplateName()
+        subfolder = dialog.getSubFolder()
 
         RS.settings.setValue('last_used_template', template_name)
+        RS.settings.setValue('last_subfolder', subfolder)
+        if self._session.isRunning():
+            RS.settings.setValue('last_session', self._session.getShortPath())
 
         if template_name:
-            self.toDaemon(
-                '/ray/server/new_from_template',
-                session_name,
-                template_name)
+            self.toDaemon('/ray/server/new_session', session_name,
+                          template_name)
         else:
             self.toDaemon('/ray/server/new_session', session_name)
 
     def openSession(self, action):
-        dialog = OpenSessionDialog(self)
+        dialog = child_dialogs.OpenSessionDialog(self)
         dialog.exec()
         if not dialog.result():
             return
 
         if self._session.isRunning():
-            RS.settings.setValue('last_session', self._session.name)
+            RS.settings.setValue('last_session', self._session.getShortPath())
 
         session_name = dialog.getSelectedSession()
         self.toDaemon('/ray/server/open_session', session_name)
 
     def closeSession(self):
-        RS.settings.setValue('last_session', self._session.name)
+        RS.settings.setValue('last_session', self._session.getShortPath())
         self.toDaemon('/ray/session/close')
 
     def abortSession(self):
-        dialog = AbortSessionDialog(self)
+        dialog = child_dialogs.AbortSessionDialog(self)
         dialog.exec()
 
         if dialog.result():
+            RS.settings.setValue('last_session', self._session.getShortPath())
             self.toDaemon('/ray/session/abort')
 
     def renameSessionAction(self):
@@ -382,7 +410,7 @@ class MainWindow(QMainWindow):
         self.ui.stackedWidgetSessionName.toggleEdit()
 
     def duplicateSession(self):
-        dialog = NewSessionDialog(self, True)
+        dialog = child_dialogs.NewSessionDialog(self, True)
         dialog.exec()
         if not dialog.result():
             return
@@ -393,18 +421,31 @@ class MainWindow(QMainWindow):
         self.toDaemon('/ray/session/duplicate', session_name)
 
     def saveTemplateSession(self):
-        dialog = SaveTemplateSessionDialog(self)
+        dialog = child_dialogs.SaveTemplateSessionDialog(self)
         dialog.exec()
         if not dialog.result():
             return
 
         session_template_name = dialog.getTemplateName()
         self.toDaemon('/ray/session/save_as_template', session_template_name)
-
+        
+    def returnToAPreviousState(self):
+        dialog = snapshots_dialog.SessionSnapshotsDialog(self)
+        dialog.exec()
+        if not dialog.result():
+            return 
+        
+        snapshot = dialog.getSelectedSnapshot()
+        self.toDaemon('/ray/session/open_snapshot', snapshot)
+    
     def aboutRaySession(self):
-        dialog = AboutRaySessionDialog(self)
+        dialog = child_dialogs.AboutRaySessionDialog(self)
         dialog.exec()
 
+    def donate(self, display_no_again=False):
+        dialog = child_dialogs.DonationsDialog(self, display_no_again)
+        dialog.exec()
+        
     def saveSession(self):
         self.toDaemon('/ray/session/save')
 
@@ -414,13 +455,13 @@ class MainWindow(QMainWindow):
                 ray.ServerStatus.OFF):
             return
 
-        dialog = AddApplicationDialog(self)
+        dialog = add_application_dialog.AddApplicationDialog(self)
         dialog.exec()
         dialog.saveCheckBoxes()
 
         if dialog.result():
-            template_name = dialog.getSelectedTemplate()
-            factory = dialog.isTemplateFactory(template_name)
+            template_name, factory = dialog.getSelectedTemplate()
+            #factory = dialog.isTemplateFactory(template_name)
             self.toDaemon(
                 '/ray/session/add_client_template',
                 int(factory),
@@ -432,56 +473,83 @@ class MainWindow(QMainWindow):
                 ray.ServerStatus.OFF):
             return
 
-        dialog = NewExecutableDialog(self)
+        dialog = child_dialogs.NewExecutableDialog(self)
         dialog.exec()
         if not dialog.result():
             return
 
-        command = dialog.getExecutableSelected()
-        proxy = dialog.runViaProxy()
-
-        if proxy:
-            self.toDaemon('/ray/session/add_proxy', command)
-        else:
-            self.toDaemon('/ray/session/add_executable', command)
+        #command = dialog.getExecutableSelected()
+        #proxy = dialog.runViaProxy()
+        
+        command, via_proxy, prefix_mode, prefix, client_id = dialog.getSelection()
+        
+        self.toDaemon('/ray/session/add_executable', command, int(via_proxy),
+                      prefix_mode, prefix, client_id)
+        
+        #if proxy:
+            #self.toDaemon('/ray/session/add_proxy', command)
+        #else:
+            #self.toDaemon('/ray/session/add_executable', command)
 
     def stopClient(self, client_id):
         client = self._session.getClient(client_id)
         if not client:
             return
+        
+        if client.no_save_level and client.check_last_save:
+            dialog = child_dialogs.StopClientNoSaveDialog(self, client_id)
+            dialog.exec()
+            if not dialog.result():
+                return
 
-        if client.status == ray.ClientStatus.READY and client.check_last_save:
+        elif (client.status == ray.ClientStatus.READY
+              and client.check_last_save):
             if client.has_dirty:
                 if client.dirty_state:
-                    dialog = StopClientDialog(self, client_id)
+                    dialog = child_dialogs.StopClientDialog(self, client_id)
                     dialog.exec()
                     if not dialog.result():
                         return
 
             # last save (or start) more than 60 seconds ago
             elif (time.time() - client.last_save) >= 10:
-                dialog = StopClientDialog(self, client_id)
+                dialog = child_dialogs.StopClientDialog(self, client_id)
                 dialog.exec()
                 if not dialog.result():
                     return
 
         self.toDaemon('/ray/client/stop', client_id)
 
-    def abortCopy(self):
-        if not self.server_copying:
+    def statusBarPressed(self):
+        status = self._session.server_status
+        
+        if status not in (
+                ray.ServerStatus.PRECOPY,
+                ray.ServerStatus.COPY,
+                ray.ServerStatus.SNAPSHOT,
+                ray.ServerStatus.OUT_SNAPSHOT,
+                ray.ServerStatus.WAIT_USER):
             return
+        
+        if status in (ray.ServerStatus.PRECOPY, ray.ServerStatus.COPY):
+            if not self.server_copying:
+                return
 
-        if self._session.server_status not in (
-                ray.ServerStatus.PRECOPY, ray.ServerStatus.COPY):
-            return
+            dialog = child_dialogs.AbortServerCopyDialog(self)
+            dialog.exec()
 
-        dialog = AbortServerCopyDialog(self)
-        dialog.exec()
+            if not dialog.result():
+                return
 
-        if not dialog.result():
-            return
-
-        self.toDaemon('/ray/server/abort_copy')
+            self.toDaemon('/ray/server/abort_copy')
+        
+        elif status in (ray.ServerStatus.SNAPSHOT,
+                        ray.ServerStatus.OUT_SNAPSHOT):
+            self.showSnapshotProgressDialog()
+            
+        elif status == ray.ServerStatus.WAIT_USER:
+            dialog = child_dialogs.WaitingCloseUserDialog(self)
+            dialog.exec()
 
     def abortCopyClient(self, client_id):
         if not self.server_copying:
@@ -492,7 +560,7 @@ class MainWindow(QMainWindow):
                 ray.ClientStatus.COPY, ray.ClientStatus.PRECOPY):
             return
 
-        dialog = AbortClientCopyDialog(self, client_id)
+        dialog = child_dialogs.AbortClientCopyDialog(self, client_id)
         dialog.exec()
 
         if not dialog.result():
@@ -502,23 +570,31 @@ class MainWindow(QMainWindow):
 
     def renameSession(self, new_session_name):
         self.toDaemon('/ray/session/rename', new_session_name)
+        
+    def addFavorite(self, name, icon_name, factory):
+        self._session.addFavorite(name, icon_name, factory, True)
+        
+    def removeFavorite(self, name, factory):
+        self._session.removeFavorite(name, factory, True)
 
-    def showClientTrashDialog(self, client_id):
-        for trashed_client in self._session.trashed_clients:
-            if trashed_client.data.client_id == client_id:
-                break
-        else:
+    def showSnapshotProgressDialog(self):
+        if self.progress_dialog_visible:
             return
-
-        dialog = ClientTrashDialog(self, trashed_client.data)
+        self.progress_dialog_visible = True
+        
+        dialog = child_dialogs.SnapShotProgressDialog(self)
+        dialog.serverProgress(self.server_progress)
         dialog.exec()
+        
+        self.progress_dialog_visible = False
+        
         if not dialog.result():
-            return
-
-        self.toDaemon('/ray/trash/restore', client_id)
-
+            return 
+        
+        self.toDaemon('/ray/server/abort_snapshot')
+        
     def showDaemonUrlWindow(self, err_code, ex_url=''):
-        dialog = DaemonUrlWindow(self, err_code, ex_url)
+        dialog = child_dialogs.DaemonUrlWindow(self, err_code, ex_url)
         dialog.exec()
         if not dialog.result():
             if (CommandLineArgs.under_nsm
@@ -539,29 +615,10 @@ class MainWindow(QMainWindow):
 
     ###FUNCTIONS RELATED TO SIGNALS FROM OSC SERVER#######
 
-    def serverAddsClient(self, client_data):
-        self._session.addClient(client_data)
-
-    def serverRemovesClient(self, client_id):
-        client = self._session.getClient(client_id)
-        if client:
-            self.ui.listWidget.removeClientWidget(client_id)
-            self._session.removeClient(client_id)
-
-    def serverSwitchesClient(self, old_client_id, new_client_id):
-        self._session.switchClient(old_client_id, new_client_id)
-
-    def serverClientProgress(self, client_id, progress):
-        client = self._session.getClient(client_id)
-        if client:
-            client.setProgress(progress)
-
-    def serverUpdatesClientProperties(self, client_data):
-        self._session.updateClientProperties(client_data)
-
-    def serverUpdatesClientStatus(self, client_id, status):
-        self._session.updateClientStatus(client_id, status)
+    def removeClient(self, client_id):
+        self.ui.listWidget.removeClientWidget(client_id)
 
+    def clientStatusChanged(self, client_id, status):
         # launch/stop flashing status if 'open'
         for client in self._session.client_list:
             if client.status == ray.ClientStatus.OPEN:
@@ -583,25 +640,11 @@ class MainWindow(QMainWindow):
                 if status == ray.ClientStatus.READY:
                     self.raiseWindow()
 
-    def serverSetsClientHasGui(self, client_id):
-        self._session.setClientHasGui(client_id)
-
-    def serverSetsClientGuiState(self, client_id, state):
-        self._session.setClientGuiState(client_id, state)
-
-    def serverSetsClientDirtyState(self, client_id, bool_dirty):
-        self._session.setClientDirtyState(client_id, bool_dirty)
-
-    def serverStillRunningClient(self, client_id):
-        self._session.clientIsStillRunning(client_id)
-
-    def serverPrintsMessage(self, message):
+    def printMessage(self, message):
         self.ui.textEditMessages.appendPlainText(
             time.strftime("%H:%M:%S") + '  ' + message)
 
-    def serverRenamesSession(self, session_name, session_path):
-        self._session.setName(session_name)
-
+    def renameSession(self, session_name, session_path):
         if session_name:
             self.setWindowTitle('%s - %s' % (ray.APP_TITLE, session_name))
             self.ui.stackedWidgetSessionName.setText(session_name)
@@ -610,33 +653,22 @@ class MainWindow(QMainWindow):
             self.ui.stackedWidgetSessionName.setText(
                 _translate('main view', 'No Session Loaded'))
 
-    def serverSessionRenameable(self, renameable):
-        self._session.is_renameable = renameable
-        
-        bool_set_edit = bool(renameable
-                             and self._session.server_status
-                                    == ray.ServerStatus.READY
-                             and not CommandLineArgs.out_daemon)
+    def setSessionNameEditable(self, bool_set_edit):
         self.ui.stackedWidgetSessionName.setEditable(bool_set_edit)
-
-    def serverSendsError(self, args):
-        if not len(args) >= 3:
-            return
-
-        error_dialog = ErrorDialog(self, args)
+        
+    def errorMessage(self, message):
+        error_dialog = child_dialogs.ErrorDialog(self, message)
         error_dialog.exec()
 
-    def serverOpensNsmSession(self):
+    def openingNsmSession(self):
         if not RS.settings.value('OpenNsmSessionInfo', True, type=bool):
             return
 
-        dialog = OpenNsmSessionInfoDialog(self)
+        dialog = child_dialogs.OpenNsmSessionInfoDialog(self)
         dialog.exec()
 
-    def serverReorderClients(self, client_id_list):
-        self._session.reOrderClients(client_id_list)
-
     def serverProgress(self, progress):
+        self.server_progress = progress
         self.ui.lineEditServerStatus.setProgress(progress)
 
     def serverCopying(self, copying):
@@ -650,11 +682,18 @@ class MainWindow(QMainWindow):
             serverStatusString(server_status))
         self.ui.frameCurrentSession.setEnabled(
             bool(server_status != ray.ServerStatus.OFF))
+        
+        if server_status in (ray.ServerStatus.SNAPSHOT,
+                             ray.ServerStatus.OUT_SNAPSHOT):
+            self.timer_snapshot.start()
+        elif self.timer_snapshot.isActive():
+            self.timer_snapshot.stop()
 
         if server_status == ray.ServerStatus.COPY:
             self.ui.actionSaveSession.setEnabled(False)
             self.ui.actionCloseSession.setEnabled(False)
             self.ui.actionAbortSession.setEnabled(False)
+            self.ui.actionReturnToAPreviousState.setEnabled(False)
             return
 
         if server_status == ray.ServerStatus.PRECOPY:
@@ -663,6 +702,7 @@ class MainWindow(QMainWindow):
             self.ui.actionAbortSession.setEnabled(True)
             self.ui.actionDuplicateSession.setEnabled(False)
             self.ui.actionSaveTemplateSession.setEnabled(False)
+            self.ui.actionReturnToAPreviousState.setEnabled(False)
             self.ui.actionAddApplication.setEnabled(False)
             self.ui.actionAddExecutable.setEnabled(False)
             self.ui.actionOpenSessionFolder.setEnabled(True)
@@ -671,23 +711,36 @@ class MainWindow(QMainWindow):
         close_or_off = bool(
             server_status in (
                 ray.ServerStatus.CLOSE,
+                ray.ServerStatus.WAIT_USER,
+                ray.ServerStatus.OUT_SAVE,
+                ray.ServerStatus.OUT_SNAPSHOT,
                 ray.ServerStatus.OFF))
         ready = bool(server_status == ray.ServerStatus.READY)
 
         self.ui.actionSaveSession.setEnabled(ready)
         self.ui.actionCloseSession.setEnabled(ready)
-        self.ui.actionAbortSession.setEnabled(not close_or_off)
+        self.ui.actionAbortSession.setEnabled(
+            not bool(server_status in (ray.ServerStatus.CLOSE,
+                                       ray.ServerStatus.OFF)))
         self.ui.actionDuplicateSession.setEnabled(not close_or_off)
+        self.ui.actionReturnToAPreviousState.setEnabled(not close_or_off)
         self.ui.actionRenameSession.setEnabled(ready)
         self.ui.actionSaveTemplateSession.setEnabled(not close_or_off)
         self.ui.actionAddApplication.setEnabled(not close_or_off)
         self.ui.actionAddExecutable.setEnabled(not close_or_off)
+        self.ui.toolButtonFavorites.setEnabled(
+            bool(self._session.favorite_list and not close_or_off))
+        self.favorites_menu.setEnabled(
+            bool(self._session.favorite_list and not close_or_off))
         self.ui.actionOpenSessionFolder.setEnabled(
             bool(server_status != ray.ServerStatus.OFF))
 
         self.ui.stackedWidgetSessionName.setEditable(
             ready and self._session.is_renameable)
         
+        self.ui.trashButton.setEnabled(bool(self._session.trashed_clients)
+                                       and not close_or_off)
+        
         if (CommandLineArgs.under_nsm
                 and not CommandLineArgs.out_daemon
                 and ready
@@ -704,12 +757,19 @@ class MainWindow(QMainWindow):
             self.ui.actionDuplicateSession.setEnabled(False)
             self.ui.actionCloseSession.setEnabled(False)
             self.ui.actionAbortSession.setEnabled(False)
-
+        
         if server_status == ray.ServerStatus.OFF:
             if self.terminate_request:
                 self._daemon_manager.stop()
+                
+        if server_status == ray.ServerStatus.WAIT_USER:
+            if not RS.settings.value(
+                    'hide_wait_close_user_dialog', False, type=bool):
+                dialog = child_dialogs.WaitingCloseUserDialog(self)
+                dialog.exec()
+            
 
-    def serverTrashAdd(self, client_data):
+    def trashAdd(self, client_data):
         prettier_name = client_data.name
         if client_data.label:
             prettier_name = client_data.label
@@ -717,38 +777,80 @@ class MainWindow(QMainWindow):
         act_x_trashed = self.trashMenu.addAction(
             QIcon.fromTheme(client_data.icon), prettier_name)
         act_x_trashed.setData(client_data.client_id)
-        act_x_trashed.triggered.connect(self._signaler.restoreClient)
-
-        trashed_client = TrashedClient(client_data, act_x_trashed)
-
-        self._session.trashed_clients.append(trashed_client)
-        self.ui.trashButton.setEnabled(True)
-
-    def serverTrashRemove(self, client_id):
+        act_x_trashed.triggered.connect(self.showClientTrashDialog)
+        
+        self.ui.trashButton.setEnabled(
+            bool(not self._session.server_status in (
+                        ray.ServerStatus.OFF,
+                        ray.ServerStatus.OUT_SAVE,
+                        ray.ServerStatus.WAIT_USER,
+                        ray.ServerStatus.OUT_SNAPSHOT,
+                        ray.ServerStatus.CLOSE)))
+        
+        return act_x_trashed
+    
+    def trashRemove(self, menu_action):
+        self.trashMenu.removeAction(menu_action)
+        
+        if not self._session.trashed_clients:
+            self.ui.trashButton.setEnabled(False)
+    
+    def trashClear(self):
+        self.trashMenu.clear()
+        self.ui.trashButton.setEnabled(False)
+    
+    @pyqtSlot()
+    def showClientTrashDialog(self):
+        try:
+            client_id = str(self.sender().data())
+        except BaseException:
+            return
+        
         for trashed_client in self._session.trashed_clients:
             if trashed_client.data.client_id == client_id:
                 break
         else:
             return
 
-        self.trashMenu.removeAction(trashed_client.menu_action)
-        self._session.trashed_clients.remove(trashed_client)
-
-        if not self._session.trashed_clients:
-            self.ui.trashButton.setEnabled(False)
-
-    def serverTrashClear(self):
-        self._session.trashed_clients.clear()
-        self.trashMenu.clear()
-        self.ui.trashButton.setEnabled(False)
+        dialog = child_dialogs.ClientTrashDialog(self, trashed_client.data)
+        dialog.exec()
+        if not dialog.result():
+            return
 
+        self.toDaemon('/ray/trash/restore', client_id)
+    
+    @pyqtSlot()
+    def launchFavorite(self):
+        template_name, factory = self.sender().data()
+        self.toDaemon('/ray/session/add_client_template',
+                      int(factory),
+                      template_name)
+        
+    def updateFavoritesMenu(self):
+        self.favorites_menu.clear()
+        
+        enable = bool(self._session.favorite_list
+                      and not self._session.server_status in (
+                        ray.ServerStatus.OFF,
+                        ray.ServerStatus.CLOSE,
+                        ray.ServerStatus.OUT_SAVE,
+                        ray.ServerStatus.OUT_SNAPSHOT))
+        
+        self.ui.toolButtonFavorites.setEnabled(enable)
+        
+        for favorite in self._session.favorite_list:
+            act_app = self.favorites_menu.addAction(
+                        ray.getAppIcon(favorite.icon, self), favorite.name)
+            act_app.setData([favorite.name, favorite.factory])
+            act_app.triggered.connect(self.launchFavorite)
+        
     def daemonCrash(self):
         QMessageBox.critical(
             self, _translate(
                 'errors', "daemon crash!"), _translate(
                 'errors', "ray-daemon crashed, sorry !"))
         QApplication.quit()
-
+        
     def saveWindowSettings(self):
         RS.settings.setValue('MainWindow/geometry', self.saveGeometry())
         RS.settings.setValue('MainWindow/WindowState', self.saveState())
@@ -761,12 +863,14 @@ class MainWindow(QMainWindow):
         RS.settings.sync()
 
     # Reimplemented Functions
-
+    
     def closeEvent(self, event):
         self.saveWindowSettings()
-
-        self.quitApp()
-        event.ignore()
+        
+        if self.quitApp():
+            QMainWindow.closeEvent(self, event)
+        else:
+            event.ignore()
 
     def leaveEvent(self, event):
         if self.isActiveWindow():
